// com.tts.initial.html5.navigation.back.history
// HTML4 / HTML5: No differences
define(["jquery", "./locals"], function (jq, uiText) {

	"use strict";

	var EventUtil = (function() {

        var
            isIOS = (/iphone|ipad|ipod/i.test( navigator.userAgent.toLowerCase() ) );

        function getFirstSupportedEvent(tagName, eventNames) {
            var result = "";
            jq.each(eventNames.split(" "), function (i, evtName) {
                if (isEventSupported(evtName, tagName)) {
                    result = evtName;
                    return false;
                }
            });
            return result;
        }

        function isEventSupported(eventName, tagName) {
            var elem = tagName ? document.createElement(tagName) : document,
                onEventName = "on" + eventName,
                isSupported;

            isSupported = !!(onEventName in elem);
            if (!isSupported && typeof elem.setAttribute !== "undefined") {
                elem.setAttribute(onEventName, "return;");
                isSupported = typeof elem[onEventName] === "function";
            }

            return isSupported;
        }

		return {
            /**
             * Attach the event handler <b>handler</b> for the first supported event specified by <b>eventNames</b>
             * to the element <b>jqObject</b>.
             * Example: attachFirstSupportedEvent( myObject, "pointerdown mousedown touchend", myHandler );
             *
             * @param element
             * @param {String} eventNames
             * @param handler
             * @return {String} the event name or ""
             */
            attachFirstSupportedEvent: function (element, eventNames, handler) {
                var jqObject = jq(element),
                    evtName = getFirstSupportedEvent(element.nodeName, eventNames);

                if (!evtName) {
                    return "";
                }

                jqObject.on(evtName, handler);
                return evtName;
            },

            /**
             * Remove attached event handler <b>handler</b> for the first supported event specified by <b>eventNames</b>
             * from the element <b>jqObject</b>.
             * Example: removeFirstSupportedEvent( myObject, "pointerdown mousedown touchend", myHandler );
             *
             * @param element
             * @param {String} eventNames
             * @param handler
             * @return {String} the event name or ""
             */
            removeFirstSupportedEvent: function (element, eventNames, handler) {
                var jqObject = jq(element),
                    evtName = getFirstSupportedEvent(element.nodeName, eventNames);

                if (!evtName) {
                    return "";
                }

                jqObject.off(evtName, handler);
                return evtName;
            },

            getPointerDownEvent: function( element ) {
                if( !isIOS ) {
                    return getFirstSupportedEvent( element.nodeName, "pointerdown mousedown touchstart" );
                }

                // iOS 9.3 unterstützt zwar click und mouseup, weshalb sich auf diese Events attached wird, aber sie werden
                //  aus unbekannten Gründen nicht gesendet.
                return "touchstart";
            },

            getPointerUpEvent: function( element ) {

                if( !isIOS ) {
                    return getFirstSupportedEvent( element.nodeName, "click pointerup mouseup touchend" );
                }

                // iOS 9.3 unterstützt zwar click und mouseup, weshalb sich auf diese Events attached wird, aber sie werden
                //  aus unbekannten Gründen nicht gesendet.
                return "touchend";
            }
        };
	})();

	var ButtonState = {
			Default : "btn_state_default",
            Hover : "btn_state_hover",
            Pressed : "btn_state_pressed",
            Inactive : "btn_state_inactive"
		},
		ImageButtonBuilder = function( options ){

            var backgroundPositions = {};
            backgroundPositions[ ButtonState.Default ] = "0 0";
            backgroundPositions[ ButtonState.Hover ] = "0 " + (-options.height) + "px";
            backgroundPositions[ ButtonState.Pressed ] = "0 " + (-2 * options.height) + "px";
            backgroundPositions[ ButtonState.Inactive ] = "0 " + (-3 * options.height) + "px";

            function createTextContent() {

                if (jq.trim(options.text.content) === "") {
                    return "";
                }

                var textContainer = "";
                if (options.text.verticalAlignment === "middle") {
                    textContainer = jq(document.createElement("div"))
                        .css({
                            "width": "auto",
                            "padding-right": options.text.paddingRight + "px",
                            "padding-left": options.text.paddingLeft + "px",
                            "vertical-align": "middle",
                            "display": "table-cell"
                        });
                }
                else {
                    textContainer = jq(document.createElement("div"))
                        .css({
                            "position": "absolute",
                            "padding": options.text.paddingTop + "px " + options.text.paddingRight + "px " + options.text.paddingBottom + "px " + options.text.paddingLeft + "px"
                        });

                    if (options.text.verticalAlignment === "bottom" || options.text.verticalAlignment === "top" ) {
                        textContainer.css(options.text.verticalAlignment, "0");
                    }

                    if( options.text.orientation === "right" ) {
                        textContainer.css(options.text.orientation, "0");
                    }
                }

                textContainer.html(options.text.content);
                return textContainer;
            }

            function createButton() {
                var currentState = ButtonState.Default,
                    imageButton = jq(document.createElement("div"))
                        .attr({
                            "title" : options.caption,
                            "class" : currentState
                        })
                        .css({
                        "width": options.width + "px",
                        "height": options.height + "px",
                        "text-align": options.text.orientation,
                        "font-family": options.text.fontFamily,
                        "color": options.color,
                        "-webkit-tap-highlight-color": "rgba(0,0,0,0)",
                        "font-size": Math.min( Math.abs(options.text.fontSize), 100) + "px",
                        "cursor": "pointer"
                    })
                    .addClass("sc-navigation")
                    .on("touchmove", function(e) {
                        e.preventDefault ? e.preventDefault() : true;
                    });

                if (options.text.verticalAlignment === "middle") {
                    imageButton.css("display", "table");
                }

                imageButton.append( createTextContent() );

                return {
                    getHotspotElement : function(){
                        return imageButton.get(0);
                    },
                    changeImage : function( width, height, filename ){

                        backgroundPositions[ ButtonState.Hover ] = "0 " + (-height) + "px";
                        backgroundPositions[ ButtonState.Pressed ] = "0 " + (-2 * height) + "px";
                        backgroundPositions[ ButtonState.Inactive ] = "0 " + (-3 * height) + "px";

                        imageButton.css({
                            "width": width + "px",
                            "height": height + "px",
                            "background-image": "url(\"" + filename + "\")",
                            "background-position": backgroundPositions[ currentState ]
                        });
                    },
                    hasState : function( buttonState ){
                        return buttonState === currentState;
                    },
                    changeState : function( buttonState ){

                        if( currentState === buttonState ){
                            return;
                        }

                        var enabled = buttonState !== ButtonState.Inactive;

                        imageButton.removeClass( currentState );
                        currentState = buttonState;
                        imageButton.addClass( buttonState );
                        imageButton.css({
                            "background-position": backgroundPositions[buttonState],
                            "cursor": enabled ? "pointer" : "default"
                        });
                    }
                };
            }

			return {
				build : createButton
			};
		},
		SvgButtonBuilder = function( options ){

			var buttonColors = {};
            buttonColors[ ButtonState.Default ] = options.colors.default;
            buttonColors[ ButtonState.Hover ] = options.colors.hover;
            buttonColors[ ButtonState.Pressed ] = options.colors.pressed;
            buttonColors[ ButtonState.Inactive ] = options.colors.inactive;

            function createSvgPath(){
                return  '<svg id="' + options.id + '" width="' + options.width + 'px" height="' + (options.height * 4) + 'px" viewBox="0 0 44 176" preserveAspectRatio="none" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" style="background-color: transparent;">' +
                            '<title>' + options.caption +'</title>' +
							'<defs></defs>' +
							'<g stroke="none" stroke-width="1" fill="none" fill-rule="evenodd">' +
                                '<g>' +
                                    '<g class="NavigationButton" transform="translate(0.000000, 0.000000)" fill="' + options.colors.default.bg + '">' +
                                        '<rect x="0" y="0" width="44" height="44"></rect>' +
                                        '<polygon class="NavigationButtonIcon" fill="' + options.colors.default.icon + '" fill-rule="nonzero" transform="translate(20.500000, 22.000000) rotate(-180.000000) translate(-20.500000, -22.000000) " points="13 12 28 21.9999994 13 32"></polygon>' +
                                    '</g>' +
                                '</g>' +
                            '</g>' +
						'</svg>';
            }

            function createButton(){

                var currentState = ButtonState.Default,
                    svgButton = jq(document.createElement("div")),
                    svg = svgButton.html( createSvgPath() )
                    .contents()
                    .addClass("sc-navigation")
                        .css({
                            "pointer-events" : "none"
                        })
                    .on("touchmove", function(e) {
                        e.preventDefault ? e.preventDefault() : true;
                    })
                    .get(0);

                var svgWrapper = jq(document.createElement("div"))
                    .attr({
                        "title" : options.caption,
                        "class" : currentState
                    })
                    .css({
                        "cursor": "pointer",
                        "width": options.width + "px",
                        "height": options.height + "px",
                        "overflow": "hidden"
                    })
                    .append(svg);

                return {
                    getHotspotElement : function(){
                        return svgWrapper.get(0);
                    },
                    hasState : function( buttonState ){
                        return buttonState === currentState;
                    },
                    changeState : function( buttonState ){

                        if( currentState === buttonState ){
                            return;
                        }

                        var colors = buttonColors[buttonState],
                            jqSvg = jq(svg);

                        svgWrapper.removeClass( currentState );
                        currentState = buttonState;
                        svgWrapper.addClass( buttonState );

                        jqSvg.find("[class=NavigationButton]").attr( "fill", colors.bg );
                        jqSvg.find("[class=NavigationButtonIcon]").attr( "fill", colors.icon );
                        jqSvg.css({
                            "cursor" : buttonState !== ButtonState.Inactive ? "pointer" : "default"
                        });
                    }
                };
            }

            return {
                build : createButton
            };
        };


	/**
	 * @constructor
	 */
	return function (context, metadata) {

		var shellElement,
			button,
			pageId = metadata.page.getId(),
            buttonInitialized = false,
			isStandardPlayer = context.getPlayerName() === "standard",
			hasStageTriggerSupport = true,
			isInAuthoringMode = false,
            pointerUpEventName, pointerDownEventName;

        function loadBackgroundImage() {

			var deferred = jq.Deferred(),
				defaultImage = new Image(),
				metadataParameters = metadata.parameters,
				backgroundImage = metadataParameters.get("bg_image", null),
				backgroundFileName,
				result = {};

			defaultImage.onload = function () {
				result.width = this.width;
				result.height = this.height / 4;
				deferred.resolve(result);
			};

			if (backgroundImage) {
				backgroundFileName = context.stage.getSharedPath() + backgroundImage.fileName;
			}
			else {
				backgroundFileName = context.getPath() + "back.png";
			}

			result.fileName = backgroundFileName;
			defaultImage.src = backgroundFileName;

			return deferred.promise();
		}

		function cancelEvent(e) {
			if(e && e.preventDefault) {
				e.preventDefault();
			}
			if(e && e.stopPropagation()) {
				e.stopPropagation();
			}
			return false;
		}

		function back() {
			var pageHistory = updateLocalHistoryArray(),
				previousPageId;
			if (pageHistory && pageHistory.length > 0) {
				pageHistory.pop();
				pageHistory = updateLocalHistoryArray(pageHistory);

				if (isHistoryEmpty()) {
					return;
				}
				previousPageId = pageHistory[pageHistory.length - 1];

				if(typeof context.stage.navigation.gotoPageById === "function") {
					context.stage.navigation.gotoPageById(previousPageId);
				} else if(typeof context.stage.navigation.gotoPage === "function") {
					context.stage.navigation.gotoPage({
						getId: function() {
							return previousPageId;
						}
					});
				}
			}
		}

		// create an object shield so we are able to select and drag element
		function createAuthoringShield(shellElement) {
			shellElement.append(jq(document.createElement("div"))
				.css({
					"position": "absolute",
					"left": 0,
					"top": 0,
					"background-color": "#ffffff",
					"opacity": "0.4",
					"width": "100%",
					"height": "100%",
					"z-index": "9999"
				}));
		}

		function lockNavigation() {
			if (isInAuthoringMode || !hasStageTriggerSupport) {
				return;
			}
			context.stage.trigger.lock(metadata.getId());
		}

		function unlockNavigation() {
			if (isInAuthoringMode || !hasStageTriggerSupport) {
				return;
			}
			context.stage.trigger.unlock(metadata.getId());
		}

		function onMouseOver() {
            if( !button.hasState( ButtonState.Pressed ) ) {
                button.changeState(ButtonState.Hover);
            }
		}

		function onMouseDown() {
			lockNavigation();
            button.changeState( ButtonState.Pressed );
		}

		function deactivateButton() {
            removeEvents();
            button.changeState( ButtonState.Inactive );
		}

		function isOnTarget(e) {
			var elem = e.target,
				origEvent = e.originalEvent,
				changedTouch = origEvent.changedTouches && origEvent.changedTouches.length > 0 ? origEvent.changedTouches[0] : undefined;

			if(changedTouch) {
				elem = document.elementFromPoint(changedTouch.clientX, changedTouch.clientY);
			}
			return e.target === elem;
		}

		function onMouseUp(e) {
			unlockNavigation();

            button.changeState( ButtonState.Default );

			if(isOnTarget(e)) {
				back();
			}
			if(!hasStageTriggerSupport) {
				if (e.originalEvent.preventDefault) {
					e.originalEvent.preventDefault();
				}
				if (e.originalEvent.stopPropagation) {
					e.originalEvent.stopPropagation();
				}
				return cancelEvent(e);
			}
		}

		function onMouseLeave() {
			unlockNavigation();
            button.changeState( ButtonState.Default );
		}

        function initPointerEventNames() {
            if( !pointerUpEventName ) {
                pointerUpEventName = EventUtil.getPointerUpEvent( button.getHotspotElement() );
                pointerDownEventName = EventUtil.getPointerDownEvent( button.getHotspotElement() );
            }
        }

        function bindEvents() {
			if (isInAuthoringMode) {
				return;
			}

			if (!button) {
				return;
			}

			removeEvents();
            EventUtil.attachFirstSupportedEvent( button.getHotspotElement(), pointerDownEventName, onMouseDown );
            EventUtil.attachFirstSupportedEvent( button.getHotspotElement(), "pointermove mousemove", onMouseOver );
            EventUtil.attachFirstSupportedEvent( button.getHotspotElement(), "pointerout mouseout", onMouseLeave );
            EventUtil.attachFirstSupportedEvent( button.getHotspotElement(), pointerUpEventName, onMouseUp );
        }

		function removeEvents() {
			unlockNavigation();
			if (!button) {
				return;
			}

            initPointerEventNames();
            EventUtil.removeFirstSupportedEvent( button.getHotspotElement(), pointerDownEventName, onMouseDown );
            EventUtil.removeFirstSupportedEvent( button.getHotspotElement(), "pointermove mousemove", onMouseOver );
            EventUtil.removeFirstSupportedEvent( button.getHotspotElement(), "pointerout mouseout", onMouseLeave );
            EventUtil.removeFirstSupportedEvent( button.getHotspotElement(), pointerUpEventName, onMouseUp );
        }

		function addHistoryPoint() {
			var pageHistory = updateLocalHistoryArray();

			if (pageHistory && pageHistory[pageHistory.length - 1] !== pageId) {
				pageHistory.push(pageId);
			}
			updateLocalHistoryArray(pageHistory);

			if(isHistoryEmpty()) {
				deactivateButton();
			} else {
				activateButton();
			}
		}

		function updateLocalHistoryArray(pH) {
			var pageHistory;

			if(pH) {
				context.stage.variable.set("tts_pageNavigationHistory", pH, "tts.pageHistory");
			}
			pH = context.stage.variable.get("tts_pageNavigationHistory", "tts.pageHistory");

			if (!pH) {
				pageHistory = [];
				context.stage.variable.set("tts_pageNavigationHistory", [], "tts.pageHistory");
			} else {
				pageHistory = [].slice.call(pH, 0);
			}
			return pageHistory;
		}

		function isHistoryEmpty() {
			var pageHistory = updateLocalHistoryArray();
			if(!pageHistory) {
				return true;
			}
			if(pageHistory.length < 1) {
				return true;
			}
			return pageHistory.length === 1 && pageHistory[0] === pageId;
		}

		function activateButton() {
			bindEvents();
			if(button) {
                button.changeState( ButtonState.Default );
			}
		}

		function initializeButton() {

			buttonInitialized = true;

            button = metadata.parameters.asBool("use_svg") ? createSvgButton() : createImageButton();

            context.page.interaction.provokesFeedback( button.getHotspotElement(), false );
            context.page.interaction.provokesAutosolve( button.getHotspotElement(), false );

            shellElement.append( button.getHotspotElement() );

			if (isHistoryEmpty()) {
				deactivateButton();
			} else {
				activateButton();
			}

            if (isInAuthoringMode) {
                createAuthoringShield( shellElement );
            }
        }

        function createImageButton(){

            var scParams = metadata.parameters,
                imageButton = new ImageButtonBuilder({
                    width: metadata.getWidth(),
                    height: metadata.getHeight(),
                    caption: uiText.get("player.navigation.button.back", metadata.stage.getContentLanguage()),
                    color : scParams.get("color", "#919191"),
                    text : {
                        content : scParams.get("text", ""),
                        fontFamily : getFontStyle( scParams.asNumber("font-family", "1") ),
                        fontSize : scParams.asNumber("font-size", "14"),
                        orientation : scParams.get("text_orientation", "center"),
                        verticalAlignment: scParams.get("text_vertical_align", "middle"),
                        paddingLeft : Math.abs(scParams.asNumber("padding-left", "10")),
                        paddingRight : Math.abs(scParams.asNumber("padding-right", "10")),
                        paddingTop : Math.abs(scParams.asNumber("padding-top", "10")),
                        paddingBottom : Math.abs(scParams.asNumber("padding-bottom", "10"))
                    }
                }).build();

            loadBackgroundImage().then( function( buttonImage ) {
                applyButtonImage( buttonImage );
            });

            return imageButton;
        }

		function createSvgButton() {

            var scParams = metadata.parameters;
            return new SvgButtonBuilder({
                id: "svg_" + pageId + "_" + metadata.getId(),
                width: metadata.getWidth(),
                height: metadata.getHeight(),
                caption: uiText.get("player.navigation.button.back", metadata.stage.getContentLanguage()),
                colors: {
                    default: {
                        icon: scParams.get("color_svg_icon_default", "#ffffff"),
                        bg: scParams.get("color_svg_background_default", "#707070")
                    },
                    hover: {
                        icon: scParams.get("color_svg_icon_hover", "#ffffff"),
                        bg: scParams.get("color_svg_background_hover", "#fd7e2f")
                    },
                    pressed: {
                        icon: scParams.get("color_svg_icon_pressed", "#ffffff"),
                        bg: scParams.get("color_svg_background_pressed", "#df7a3b")
                    },
                    inactive: {
                        icon: scParams.get("color_svg_icon_inactive", "#a9a9a9"),
                        bg: scParams.get("color_svg_background_inactive", "#707070")
                    }
                }
            }).build();
        }

        function getFontStyle( paramValue ) {
            switch( paramValue ) {
                case 2:
                    return "\"Times New Roman\", Times, serif";
                case 3:
                    return "Verdana, Geneva, sans-serif";
                case 4:
                    return "\"Courier New\", Courier, monospace";
                default:
                    return "Arial, Helvetica, sans-serif";
            }
        }

        function applyButtonImage( buttonImage ){

            var buttonWidth = buttonImage.width,
                buttonHeight = buttonImage.height;

            shellElement.css({
                "width": buttonWidth + "px",
                "height": buttonHeight + "px"
            });

            button.changeImage( buttonWidth, buttonHeight, buttonImage.fileName );
        }

		// revealing public API
		return {
			onExitPage: function () {
				removeEvents();
			},

			onLoading: function () {
				context.page.event.bindOnEnter(this);
				hasStageTriggerSupport = !!context.stage.trigger;
				isInAuthoringMode = !!context.stage.isInAuthoringMode();

				shellElement = jq(context.getElement());

				// because of some problems with loading SVG while it is hidden on page,
				// we initialize SVG button onEnterPage
				if (!isStandardPlayer || isInAuthoringMode) {
					initializeButton();
				}
			},

			onEnterPage: function () {
				// because of some problems with loading SVG while it is hidden on page,
				// we initialize SVG button onEnterPage
				if (!buttonInitialized) {
					initializeButton();
				}

				unlockNavigation();
				addHistoryPoint();
			}
		};
	};
});
