define( function() {

    var fallbackLanguage = "en-us",
        texts = {
    "player": {
        "navigation": {
            "button": {
                "close": {
                    "et-ee": "Sulge",
                    "bg-bg": "Прекрати",
                    "cs-cz": "Ukončit",
                    "da-dk": "Afslut",
                    "de-de": "Beenden",
                    "en-us": "Close",
                    "es-es": "Cerrar",
                    "fi-fi": "Lopeta",
                    "fr-fr": "Fermer",
                    "el-gr": "Έξοδος",
                    "hi-in": "बंद करें",
                    "hr-hr": "Završi",
                    "hu-hu": "Befejezés",
                    "it-it": "Chiudi",
                    "ja-jp": "終了する",
                    "ko-kr": "닫기",
                    "lt-lt": "Užverti",
                    "mk-mk": "Затвори",
                    "mn-mn": "Хаах",
                    "ms-id": "Tutup",
                    "nl-nl": "Afsluiten",
                    "no-no": "Lukk",
                    "pl-pl": "Zakończ",
                    "pt-pt": "Terminar",
                    "ro-ro": "Închidere",
                    "ru-ru": "Выход",
                    "sk-sk": "Ukončiť",
                    "sl-si": "Končaj",
                    "sr-rs": "Izlaz",
                    "sv-se": "Stäng",
                    "th-th": "ปิด",
                    "tr-tr": "Bitir",
                    "zh-tw": "關閉",
                    "zh-cn": "关闭",
                    "de-at": "Beenden",
                    "de-ch": "Beenden",
                    "eo-de": "Beenden",
                    "nl-be": "Afsluiten",
                    "en-ie": "Close",
                    "en-gb": "Close",
                    "eo-en": "Beenden",
                    "fr-be": "Fermer"
                }
            }
        }
    }
};

    /**
     * Gets data from a nested object
     *
     * @private
     * @param {string} key - a dot separated representation of the objectstructure e.g.("cmi.core.lesson_status")
     * @param {object|string} data - the data object where the value should be got from
     */
    function getData( key, data ) {

        var parts = key.split("."),
            i = 0,
            parent = data;

        for(; i < parts.length; ++i) {
            if (!parent[parts[i]]) {
                return null;
            }
            parent = parent[parts[i]];
        }

        return parent;
    }

    // revealing public API
    return {

        /**
         * Gets a text in a given language
         * If the text is not defined in the given language, the text in teh fallback "en-us" is used, otherwise "null".
         *
         * @public
         * @param {string} key - a dot separated representation of the resource e.g.("quiz.evaluate")
         * @param {string} [language] - the language for the text e.g.("en-us")
         */
        get : function( key, language ){

            var text = getData( key, texts );
            if( text ){
                return getData( language, text ) || getData( fallbackLanguage, text );
            }

            return null;
        }
    }
});