define( function() {

    var fallbackLanguage = "en-us",
        texts = {
    "player": {
        "navigation": {
            "button": {
                "next": {
                    "et-ee": "Järgmine",
                    "bg-bg": "Продължи",
                    "cs-cz": "Dále",
                    "da-dk": "Næste",
                    "de-de": "Weiter",
                    "en-us": "Next",
                    "es-es": "Siguiente",
                    "fi-fi": "Seuraava",
                    "fr-fr": "Suivant",
                    "el-gr": "Επόμενο",
                    "hi-in": "आगे",
                    "hr-hr": "Dalje",
                    "hu-hu": "Tovább",
                    "it-it": "Avanti",
                    "ja-jp": "詳細",
                    "ko-kr": "다음",
                    "lt-lt": "Tolesnis",
                    "mk-mk": "Следно",
                    "mn-mn": "Дараах",
                    "ms-id": "Berikutnya",
                    "nl-nl": "Verder",
                    "no-no": "Neste",
                    "pl-pl": "Dalej",
                    "pt-pt": "Avançar",
                    "ro-ro": "Mai departe",
                    "ru-ru": "Дальше",
                    "sk-sk": "Ďalej",
                    "sl-si": "Naprej",
                    "sr-rs": "Dalje",
                    "sv-se": "Nästa",
                    "th-th": "ถัดไป",
                    "tr-tr": "Devam",
                    "zh-tw": "下一步",
                    "zh-cn": "下一步",
                    "de-at": "Weiter",
                    "de-ch": "Weiter",
                    "eo-de": "Weiter",
                    "nl-be": "Verder",
                    "en-ie": "Next",
                    "en-gb": "Next",
                    "eo-en": "Weiter",
                    "fr-be": "Suivant"
                }
            }
        }
    }
};

    /**
     * Gets data from a nested object
     *
     * @private
     * @param {string} key - a dot separated representation of the objectstructure e.g.("cmi.core.lesson_status")
     * @param {object|string} data - the data object where the value should be got from
     */
    function getData( key, data ) {

        var parts = key.split("."),
            i = 0,
            parent = data;

        for(; i < parts.length; ++i) {
            if (!parent[parts[i]]) {
                return null;
            }
            parent = parent[parts[i]];
        }

        return parent;
    }

    // revealing public API
    return {

        /**
         * Gets a text in a given language
         * If the text is not defined in the given language, the text in teh fallback "en-us" is used, otherwise "null".
         *
         * @public
         * @param {string} key - a dot separated representation of the resource e.g.("quiz.evaluate")
         * @param {string} [language] - the language for the text e.g.("en-us")
         */
        get : function( key, language ){

            var text = getData( key, texts );
            if( text ){
                return getData( language, text ) || getData( fallbackLanguage, text );
            }

            return null;
        }
    }
});