// com.tts.initial.html5.play.modes
define([
        "jquery",
        "./event.utils",
        "./menu.button.builder",
        "css!./component.css"
    ],
    function(
        jq,
        EventUtil,
        MenuBuilder
    ){

    "use strict";

    /**
     * @constructor
     */

    return function (context, metadata) {

        var menuButton,
            isMenuOpen = false,
            pointerUpEventName,
            pointerDownEventName;

        function lockNavigation() {
            if (context.stage.isInAuthoringMode() || (typeof(context.stage.trigger) === "undefined")) {
                return;
            }
            context.stage.trigger.lock( metadata.getId() );
        }

        function unlockNavigation() {
            if (context.stage.isInAuthoringMode() || (typeof(context.stage.trigger) === "undefined")) {
                return;
            }
            context.stage.trigger.unlock( metadata.getId() );
        }

        function isOnTarget(e) {
            var elem = e.target,
                origEvent = e.originalEvent,
                changedTouch = origEvent.changedTouches && origEvent.changedTouches.length > 0 ? origEvent.changedTouches[0] : undefined;

            if (changedTouch) {
                elem = document.elementFromPoint(changedTouch.clientX, changedTouch.clientY);
            }

            return e.target === elem;
        }

        function onPointerDown(e){

            e.preventDefault();
            e.stopPropagation();

            lockNavigation();
            menuButton.onPointerDown();
        }

        function onPointerUp(e) {

            e.preventDefault();
            e.stopPropagation();

            unlockNavigation();

            menuButton.onHover();

            if (!isOnTarget(e)) {
                return;
            }

            var menuContainer = jq( menuButton.getMenuContainerElement() );
            if (isMenuOpen) {
                jq(menuContainer).css("display", "none");
                isMenuOpen = false;
            }
            else {
                jq(menuContainer).css("display", "block");
                isMenuOpen = true;
            }
        }

        function initPointerEventNames() {
            if( !pointerUpEventName ) {
                pointerUpEventName = EventUtil.getPointerUpEvent( menuButton.getMenuButtonElement() );
                pointerDownEventName = EventUtil.getPointerDownEvent( menuButton.getMenuButtonElement() );
            }
        }

        function bindEvents() {

            if (context.stage.isInAuthoringMode()) {
                return;
            }

            removeEvents();
            EventUtil.attachFirstSupportedEvent( menuButton.getMenuButtonElement(), pointerDownEventName, onPointerDown );
            EventUtil.attachFirstSupportedEvent( menuButton.getMenuButtonElement(), "pointermove mousemove", menuButton.onHover );
            EventUtil.attachFirstSupportedEvent( menuButton.getMenuButtonElement(), "pointerout mouseout", menuButton.onReleased  );
            EventUtil.attachFirstSupportedEvent( menuButton.getMenuButtonElement(), pointerUpEventName, onPointerUp );
        }

        function removeEvents() {

            initPointerEventNames();
            EventUtil.removeFirstSupportedEvent( menuButton.getMenuButtonElement(), pointerDownEventName, onPointerDown );
            EventUtil.removeFirstSupportedEvent( menuButton.getMenuButtonElement(), "pointermove mousemove", menuButton.onHover );
            EventUtil.removeFirstSupportedEvent( menuButton.getMenuButtonElement(), "pointerout mouseout", menuButton.onReleased );
            EventUtil.removeFirstSupportedEvent( menuButton.getMenuButtonElement(), pointerUpEventName, onPointerUp );
        }

        function initializePlayMode(){

            var defaultPlayMode = metadata.stage.getDefaultPlayMode(),
                individualPlayMode = metadata.page.getIndividualPlayMode(),
                playMode = individualPlayMode !== "none" ? individualPlayMode : defaultPlayMode;

            if (playMode === "presentation") {
                menuButton.setPresentationModeUI();
                return;
            }

            if (playMode === "film") {
                menuButton.setFilmModeUI();
                return;
            }

            menuButton.setInteractiveModeUI();
        }

        // revealing public API
        return {

            onLoading: function () {

                context.page.event.bindOnEnter(this);
                context.page.event.bindOnExit(this);
                context.stage.event.bindOnPlayModeChanged(this);

                menuButton = new MenuBuilder( context, metadata ).build();
            },

            onPlayModeChanged: function (dataHTML5, dataHTML4) {

                var params = dataHTML4 || dataHTML5,
                    playMode = params.mode;

                if (playMode === "presentation") {
                    menuButton.setPresentationModeUI();
                    return;
                }

                if (playMode === "film") {
                    menuButton.setFilmModeUI();
                    return;
                }

                menuButton.setInteractiveModeUI();
            },

            onEnterPage: function() {

                if( context.stage.isInPreviewMode() ) {
                    initializePlayMode();
                }

                bindEvents();
            },

            onExitPage: function() {
                removeEvents();
            }
        };
    };
});
