define([
    "jquery",
    "./event.utils",
    "./locals",
],
function(
    jq,
    EventUtil,
    uiText
){

    "use strict";

    /**
     * @constructor
     */

     return function( context, metadata ){

        var button,
            menuContainer,
            contentLanguage = metadata.stage.getContentLanguage(),
            BACKGROUND_POSITION;

        function loadBackgroundImage() {

            var deferred = jq.Deferred(),
                defaultImage = new Image(),
                metadataParameters = metadata.parameters,
                backgroundImage = metadataParameters.get("bg_image", null),
                backgroundFileName,
                result = {};

            defaultImage.onload = function () {
                result.width = this.width;
                result.height = this.height / 4;
                deferred.resolve(result);
            };
            if (backgroundImage) {
                backgroundFileName = context.stage.getSharedPath() + backgroundImage.fileName;
            }
            else {
                backgroundFileName = context.getPath() + "icn_view-mode.tts-a.png";
            }
            result.fileName = backgroundFileName;
            defaultImage.src = backgroundFileName;

            return deferred.promise();
        }

        function createMenuButton()  {

            var shellElement = jq(context.getElement()),
                buttonWidth = 45,
                buttonHeight = 45;

            BACKGROUND_POSITION = {
                "DEFAULT": "0 0",
                "HOVER": "0 " + (-buttonHeight) + "px",
                "PRESSED": "0 " + (-2 * buttonHeight) + "px",
                "INACTIVE": "0 " + (-3 * buttonHeight) + "px"
            };

            shellElement.css({
                "width": buttonWidth + "px",
                "height": buttonHeight + "px"
            });

            button = jq(document.createElement("div"))
                .css({
                    "width": buttonWidth + "px",
                    "height": buttonHeight + "px",
                    "cursor" : "pointer"
                })
                .addClass("sc-menu")
                .on("touchmove", function (e) {
                    e.preventDefault ? e.preventDefault() : true;
                });

			// Disabling ability to solve input and key interactions and provoking feedback
			context.page.interaction.provokesFeedback( button.get(0), false );
			context.page.interaction.provokesAutosolve( button.get(0), false );

            menuContainer = createMenuContainer( 45 );
            button.append( menuContainer.getContainerElement() );
            shellElement.append( button );

            createMenuContent( menuContainer );
        }

        function applyBackgroundImage( backgroundImage )  {

            var shellElement = jq(context.getElement()),
                buttonWidth = backgroundImage.width,
                buttonHeight = backgroundImage.height,
                imgSrc = backgroundImage.fileName;

            BACKGROUND_POSITION = {
                "DEFAULT": "0 0",
                "HOVER": "0 " + (-buttonHeight) + "px",
                "PRESSED": "0 " + (-2 * buttonHeight) + "px",
                "INACTIVE": "0 " + (-3 * buttonHeight) + "px"
            };

            shellElement.css({
                "width": buttonWidth + "px",
                "height": buttonHeight + "px"
            });

            button.css({
                "width": buttonWidth + "px",
                "height": buttonHeight + "px",
                "background-image": "url(\"" + imgSrc + "\")",
                "background-position": BACKGROUND_POSITION.DEFAULT
            });
        }

        function createMenuContainer( height ) {

            var items = [],
                container = jq(document.createElement("div"))
                    .attr({
                        "id": "menuContainer"
                    })
                    .css({
                        "margin-top": (-height) * items.length + "px",
                        "width": "240px"
                    })
                    .addClass("sc-menu-expand");

            return {
                getContainerElement : function(){
                    return container;
                },
                add : function( item ){
                    items.push( item );
                    container
                        .css({
                            "margin-top": (-height) * items.length + "px"
                        })
                        .append( item.getElement() );
                },
                getItems : function(){
                    return items;
                }
            }
        }

        function createMenuContent( menuContainer ) {

            var availablePlayModes = metadata.stage.getAvailablePlayModes(),
                individualPlayMode = metadata.page.getIndividualPlayMode();

            if ((individualPlayMode === "none" || individualPlayMode === "interactive") && availablePlayModes.interactive) {
                menuContainer.add( createMenuItem( "int", uiText.get("player.navigation.modes.interactive", contentLanguage), context.stage.navigation.switchToInteractiveMode ) );
            }

            if ((individualPlayMode === "none" || individualPlayMode === "presentation") && availablePlayModes.presentation) {
                menuContainer.add( createMenuItem("pres", uiText.get("player.navigation.modes.presentation", contentLanguage), context.stage.navigation.switchToPresentationMode ) );
            }

            if ((individualPlayMode === "none" || individualPlayMode === "film") && availablePlayModes.film) {
                menuContainer.add( createMenuItem("film", uiText.get("player.navigation.modes.film", contentLanguage), context.stage.navigation.switchToFilmMode ) );
            }
        }

        function createMenuItem( id, caption, action ) {

            var parameters = metadata.parameters,
                currModeImg = context.getPath() + "marker.png",
                itemElement,
                captionElement,
                checkBoxElement;

            itemElement = jq(document.createElement("div"))
                .attr( "id", id + "MenuItem" )
                .css({
                    "height": "40px",
                    "width": "100%",
                    "background-color": parameters.get("bg_colour", "#A6A6A6"),
                    "color": parameters.get("text_colour", "#FFFFFF"),
                    "border-bottom": "4px solid " + parameters.get("bg_colour_hover", "#FD7E2F")
                    //"font-family": "Arial, Helvetica, sans-serif",
                    //"font-size": "14px"
                })
                .addClass("sc-" + id + "-mode-button");

            itemElement.on( EventUtil.getPointerUpEvent( itemElement.get(0) ), action );

            captionElement = jq(document.createElement("span"))
                .addClass("sc-" + id + "-mode")
                .css({
                    "pointer-events" : "none"
                })
                .text( caption );

            checkBoxElement = jq(document.createElement("img"))
                .attr("src", currModeImg)
                .addClass( id + "-mode-marker");

            itemElement.append( captionElement );
            itemElement.append( checkBoxElement );

            // Setting the hover over colour
            jq(itemElement).hover(
                function(){
                    jq(this).css( "background-color", parameters.get("bg_colour_hover", "#FD7E2F") );
                },
                function() {
                    jq(this).css( "background-color", parameters.get("bg_colour", "#A6A6A6") );
                });

			// Disabling ability to solve input and key interactions and provoking feedback
            context.page.interaction.provokesFeedback( button.get(0), false );
            context.page.interaction.provokesAutosolve( button.get(0), false );

            return {
                getId : function(){
                    return id;
                },
                getElement : function(){
                    return itemElement;
                },
                getCheckBoxElement : function(){
                    return checkBoxElement;
                }
            }
        }

        // create an object shield so we are able to select and drag element
        function createAuthoringShield(shellElement) {
            shellElement.append(jq(document.createElement("div"))
                .css({
                    "position": "absolute",
                    "left": 0,
                    "top": 0,
                    "background-color": "transparent",
                    "opacity": "0.4",
                    "width": "100%",
                    "height": "100%",
                    "z-index": "9999"
                }));
        }

        function uncheckAll() {
            var items = menuContainer.getItems();
            jq.each( items, function(index,item){
                item.getCheckBoxElement().hide();
            });
        }

        function checkItem( id ){
            setTimeout( function(){
                var items = menuContainer.getItems();
                jq.each( items, function(index,item){
                    if( item.getId() === id ){
                        uncheckAll();
                        item.getCheckBoxElement().show();
                        return false; //jq.each break
                    }

                    return true; // jq.each continue
                });
            }, 50 );
        }

        return {
            build : function(){

                createMenuButton();

                loadBackgroundImage().then( function( bgImage ) {
                    applyBackgroundImage( bgImage );
                });

                if (context.stage.isInAuthoringMode()) {
                    createAuthoringShield(jq(context.getElement()));
                }

                return {
                    getMenuButtonElement : function(){
                        return button.get(0);
                    },

                    getMenuContainerElement : function(){
                        return menuContainer.getContainerElement().get(0);
                    },

                    onReleased : function(){
                        button.css("background-position", BACKGROUND_POSITION.DEFAULT);
                    },
                    onHover : function(){
                        button.css("background-position", BACKGROUND_POSITION.HOVER);
                    },
                    onPointerDown : function(){
                        button.css("background-position", BACKGROUND_POSITION.PRESSED);
                    },

                    setInteractiveModeUI : function() {
                        checkItem( "int" );
                    },
                    setPresentationModeUI : function() {
                        checkItem( "pres" );
                    },
                    setFilmModeUI : function() {
                        checkItem( "film" );
                    }
                };
            }
        }
    };
});
