// com.tts.initial.html5.quiz.ddsource.autoform
define(["jquery"], function( jq ) {

    "use strict";

    var EventUtil = (function() {

        var
            isIOS = (/iphone|ipad|ipod/i.test( navigator.userAgent.toLowerCase() ) );

        function getFirstSupportedEvent(tagName, eventNames) {
            var result = "";

            jq.each(eventNames.split(" "), function (i, evtName) {
                if (isEventSupported(evtName, tagName)) {
                    result = evtName;
                    return false;
                }
            });
            return result;
        }

        function isEventSupported(eventName, tagName) {
            var elem = tagName ? document.createElement(tagName) : document,
                onEventName = "on" + eventName,
                isSupported;

            isSupported = !!(onEventName in elem);
            if (!isSupported && typeof elem.setAttribute !== "undefined") {
                elem.setAttribute(onEventName, "return;");
                isSupported = typeof elem[onEventName] === "function";
            }

            return isSupported;
        }

        return {
            /**
             * Attach the event handler <b>handler</b> for the first supported event specified by <b>eventNames</b>
             * to the element <b>jqObject</b>.
             * Example: attachFirstSupportedEvent( myObject, "pointerdown mousedown touchend", myHandler );
             *
             * @param element
             * @param {String} eventNames
             * @param handler
             * @return {String} the event name or ""
             */
            attachFirstSupportedEvent: function (element, eventNames, handler) {
                var jqObject = jq(element),
                    evtName = getFirstSupportedEvent(element.nodeName, eventNames);

                if (!evtName) {
                    return "";
                }

                jqObject.on(evtName, handler);
                return evtName;
            },

            /**
             * Remove attached event handler <b>handler</b> for the first supported event specified by <b>eventNames</b>
             * from the element <b>jqObject</b>.
             * Example: removeFirstSupportedEvent( myObject, "pointerdown mousedown touchend", myHandler );
             *
             * @param element
             * @param {String} eventNames
             * @param handler
             * @return {String} the event name or ""
             */
            removeFirstSupportedEvent: function (element, eventNames, handler) {
                var jqObject = jq(element),
                    evtName = getFirstSupportedEvent(element.nodeName, eventNames);

                if (!evtName) {
                    return "";
                }

                jqObject.off(evtName, handler);
                return evtName;
            },

            getPointerDownEvent: function( element ) {
                if( !isIOS ) {
                    return getFirstSupportedEvent( element.nodeName, "pointerdown mousedown touchstart" );
                }

                // iOS 9.3 unterstützt zwar click und mouseup, weshalb sich auf diese Events attached wird, aber sie werden
                //  aus unbekannten Gründen nicht gesendet.
                return "touchstart";
            },

            getPointerMoveEvent: function( element ) {
                if( !isIOS ) {
                    return getFirstSupportedEvent( element.nodeName, "pointermove mousemove touchmove" );
                }

                // iOS 9.3 unterstützt zwar click und mouseup, weshalb sich auf diese Events attached wird, aber sie werden
                //  aus unbekannten Gründen nicht gesendet.
                return "touchmove";
            },

            getPointerUpEvent: function( element ) {

                if( !isIOS ) {
                    return getFirstSupportedEvent( element.nodeName, "pointerup mouseup touchend" );
                }

                // iOS 9.3 unterstützt zwar click und mouseup, weshalb sich auf diese Events attached wird, aber sie werden
                //  aus unbekannten Gründen nicht gesendet.
                return "touchend";
            }
        };
    })();

    function SourceElementBuilder( context, metadata, offsetService ){

        var	parameters = metadata.parameters,
            isStandardPlayer = context.getPlayerName() === "standard",

            colors = getColors(),

            positioningDiv = jq(context.getPositioningElement()),
            backgroundImage,

            cssIndexAlwaysOnTop = metadata.parameters.asBool( "z_index_always_on_top", true ),
            cssIndexForDragging = "1000",
            cssIndexMemory = null,

            fonts = [
                'Arial, Helvetica, sans-serif',
                '"Times New Roman", Times, serif',
                'Verdana, Geneva, sans-serif',
                '"Courier New", Courier, monospace',
                'Roboto, "Helvetica Neue", Helvetica, Arial, sans-serif',
                '"Droid Serif", "Times New Roman", Times, serif'
            ];

        function getFont(index) {
            var fontIndex = (index > 0 && index < fonts.length) ? index - 1 : 0;
            return fonts[fontIndex];
        }

        function getCoreStyles() {
            return {
                "cursor": "default",
                "width": metadata.getWidth() + "px",
                "height":  metadata.getHeight() + "px"
            };
        }

        function createStartArea() {

            var borderSize = parameters.asNumber("border-size", 2),
                startDiv = jq(document.createElement("div"))
                    .addClass("drag_start_element")
                    .css({
                        "width": (metadata.getWidth() - (2*borderSize)) + "px",
                        "height":  (metadata.getHeight() - (2*borderSize)) + "px",
                        "position": "absolute",
                        //"border" : context.stage.isInAuthoringMode() ? "0" : "1px dotted #ff0000",
                        "z-index" : "1"
                    }),
                positioningElement = jq(context.getPositioningElement()),
                pointerDownEvent = EventUtil.getPointerDownEvent( positioningElement.get(0) ),
                onFirstPointerDown = function(){
                    startDiv.offset( positioningElement.offset() );
                    positioningElement.off( pointerDownEvent, onFirstPointerDown );
                };

            positioningElement.on( pointerDownEvent, onFirstPointerDown );

            return startDiv;
        }

        function createDragSourceElement() {

            var fontFamily = parameters.asNumber("font-family", 0),
                fontSize = parameters.get("font-size", "14") + "px",
                borderSize = parameters.asNumber("border-size", 2),
                dragDiv = jq(document.createElement("div"));

            dragDiv.addClass("drag_element")
                .html( parameters.get("text", ""))
                .css({
                    "width": metadata.getWidth() + "px",
                    "height":  metadata.getHeight() + "px",
                    "position": "absolute",
                    "cursor" : "pointer",
                    "z-index" : "1",
                    "font-family": getFont(fontFamily),
                    "font-size": fontSize,
                    "font-weight": "bold",
                    "font-style": "normal",
                    "color": parameters.get("color", "#919191"),
                    "line-height": fontSize,
                    "padding-left": (parameters.asNumber("padding-left", 15) - borderSize) + "px",
                    "padding-right": (parameters.asNumber("padding-right", 15) - borderSize) + "px",
                    "padding-top": (parameters.asNumber("padding-top", 15) - borderSize) + "px",
                    "text-align": parameters.get("text_orientation", "center"),
                    "background-color": colors.default.background,
                    "box-sizing": "border-box",
                    "border" : (borderSize !== 0) ? borderSize + "px solid " + colors.default.border : 0,
                    "-webkit-tap-highlight-color": "rgba(0,0,0,0)"
                });

            backgroundImage = createBackgroundImageModel( dragDiv );

            return dragDiv;
        }

        function setZIndexForDragging(){

            if( !cssIndexAlwaysOnTop && !cssIndexMemory ){
                cssIndexMemory = positioningDiv.css("z-index");
            }

            positioningDiv.css({
                "z-index" :  cssIndexForDragging
            });
        }

        function restoreZIndex(){

            if( cssIndexAlwaysOnTop || !cssIndexMemory ){
                return;
            }

            positioningDiv.css({
                "z-index" : cssIndexMemory
            });

            cssIndexMemory = null;
        }

        function getColors() {
            return {
                "default" : {
                    "border" : asRgbaValue("border-color", "#cccccc", "border-transparency"),
                    "background" : asRgbaValue("background-color", "#cccccc", "background-transparency")
                },
                "solvedCorrect" : {
                    "border" : asRgbaValue("border-color-correct", "#74b65b", "border-transparency"),
                    "background" : asRgbaValue("background-color-correct", "#cccccc", "background-transparency")
                },
                "solvedWrong" : {
                    "border" : asRgbaValue("border-color-incorrect", "#74b65b", "border-transparency"),
                    "background" : asRgbaValue("background-color-incorrect", "#cccccc", "background-transparency")
                },
                "bestPractice" : {
                    "border" : asRgbaValue("border-color-best-solution", "#74b65b", "border-transparency"),
                    "background" : asRgbaValue("background-color-best-solution", "#cccccc", "background-transparency")
                }
            };
        }

        function asRgbaValue( colorParameterName, defaultColor, transparencyParameterName ) {

            var color = hexToRgb( parameters.get(colorParameterName, defaultColor)),
                alpha = asAlpha( parameters.asNumber(transparencyParameterName, 0) );

            return "rgba(" + color.r + ", " + color.g + ", " + color.b + ", " + alpha + ")";
        }

        function hexToRgb( hex ) {

            var parsingResult = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
            if( parsingResult ) {
                return {
                    r: parseInt(parsingResult[1], 16),
                    g: parseInt(parsingResult[2], 16),
                    b: parseInt(parsingResult[3], 16)
                };
            }

            return {
                r: 0,
                g: 0,
                b: 0
            };
        }

        function asAlpha( percentValue ) {
            return (100 - percentValue) / 100;
        }

        function createBackgroundImageModel( jqElement ) {

            var backgroundImage = metadata.parameters.get("bg_image", null),
                backgroundFileName = "",
                bgHeight = 0,
                bgLayer = jqElement;

            if (backgroundImage) {
                backgroundFileName = context.stage.getSharedPath() + backgroundImage.fileName;
                bgHeight = backgroundImage.height / 7;

                if( backgroundFileName && backgroundFileName.length > 0 ){
                    bgLayer.css({
                        "background-image": "url(\"" + backgroundFileName + "\")",
                        "background-position" :"0px 0px"
                    });
                }
            }

            return {
                setDefaultStyle : function(){
                    bgLayer.css("background-position", "0px 0px");
                },
                setMouseOverStyle : function(){
                    bgLayer.css("background-position", "0px -" + bgHeight + "px");
                },
                setDragStyle : function(){
                    bgLayer.css("background-position", "0px -" + 2 * bgHeight + "px");
                },
                setOnTargetStyle : function(){
                    bgLayer.css("background-position", "0px -" + 3 * bgHeight + "px");
                },
                setCorrectStyle : function(){
                    bgLayer.css("background-position", "0px -" + 4 * bgHeight + "px");
                },
                setWrongStyle : function(){
                    bgLayer.css("background-position", "0px -" + 5 * bgHeight + "px");
                },
                setBestPracticeStyle : function(){
                    bgLayer.css({
                        "cursor": "default",
                        "background-position": "0px -" + 6 * bgHeight + "px"
                    });
                }
            };
        }

        function reorderChildrenZIndex(){

            var children = getChildren(),
                jqChild;

            jq.each( children, function( index, childElement ){

                jqChild = jq( childElement );
                jqChild.css({
                    "z-index" : "10",
                    "pointer-events" : "none"
                });
            });
        }

        // Workaround for the HTML5-Player api method "context.getChildren()" failure. This will be fixed in TTKF 18R2
        // The shell element should not be contained in the child list.
        // In the fixed version context.getChildren() can be called directly.
        function getChildren(){

            var shellElementId = jq(context.getElement()).attr("id"),
                children = context.getChildren(),
                cleanedChildren = [];

            if( isStandardPlayer ){
                return children;
            }

            jq.each( children, function( index, childElement ){

                var jqChild = jq( childElement );

                if( jqChild.find( "#" + shellElementId ).get(0) ) {
                    return true; // jq.each continue
                }

                cleanedChildren.push( jqChild.get(0) );
                return true; // jq.each continue
            });

            return cleanedChildren;
        }

        return {
            build : function(){

                reorderChildrenZIndex();

                var shellElement = jq(context.getElement()),
                    positioningElement = jq(context.getPositioningElement()),
                    scStage = positioningElement.parent(),
                    startDiv = createStartArea(),
                    sourceUIDiv = createDragSourceElement(),
                    animating = false;

                scStage.append( startDiv );
                startDiv.offset( positioningElement.offset() );

                shellElement
                    .css( getCoreStyles() )
                    .append( sourceUIDiv );

                if( cssIndexAlwaysOnTop ){
                    positioningDiv.css({
                        "z-index" :  cssIndexForDragging
                    });
                }

                removePointerEvents();

                function removePointerEvents(){
                    positioningDiv.css({"pointer-events" : "none"});
                    shellElement.css({"pointer-events" : "none"});
                    startDiv.css({"pointer-events" : "none"});
                    sourceUIDiv.css({"pointer-events" : "auto"});
                }

                function moveToOffset ( offset, skipAnimation ) {
                    var duration = 400;

                    if( skipAnimation ){
                        duration = 0;
                    }

                    animating = true;

                    positioningElement.animate( offset, {
                        "duration": duration,
                        "easing": "swing",
                        "queue": false,
                        "complete": function () {
                            animating = false;
                        }
                    });
                }

                return {
                    initialize : function(){
                        animating = false;
                    },
                    isInAnimation : function(){
                        return animating;
                    },
                    setOffset : function( offset ){
                        positioningElement.offset(offset);
                    },
                    moveToOffset : function( offset ){
                        moveToOffset( offsetService.removeGroupOffset( offset, startDiv ) );
                    },
                    moveToStartOffset : function( skipAnimation ) {
                        var startOffset = offsetService.getRelativeStageOffset(startDiv);
                        startOffset = offsetService.removeGroupOffset( startOffset, startDiv );
                        moveToOffset( startOffset, skipAnimation );
                    },

                    getDragDiv : function() {
                        return positioningElement;
                    },

                    setDragStyle : function(){
                        setZIndexForDragging();
                        backgroundImage.setDragStyle();
                    },
                    setDroppedStyle : function( isOnTarget ){

                        sourceUIDiv.css({
                            "cursor": "pointer"
                        });

                        isOnTarget ? backgroundImage.setOnTargetStyle() : backgroundImage.setDefaultStyle();

                        restoreZIndex();
                    },
                    clearCursorStyle : function(){
                        sourceUIDiv.css({
                            "cursor" : "pointer !important"
                        });
                    },
                    setUserSolutionStyle : function( isDroppedCorrectly ) {

                        sourceUIDiv.css({
                            "cursor": "default",
                            "border-color": isDroppedCorrectly ? colors.solvedCorrect.border : colors.solvedWrong.border,
                            "background-color": isDroppedCorrectly ? colors.solvedCorrect.background : colors.solvedWrong.background
                        });

                        isDroppedCorrectly ? backgroundImage.setCorrectStyle() : backgroundImage.setWrongStyle();
                    },
                    setBestPracticeSolutionStyle : function(){

                        sourceUIDiv.css({
                            "cursor": "default",
                            "border-color" : colors.bestPractice.border,
                            "background-color" : colors.bestPractice.background
                        });
                        backgroundImage.setBestPracticeStyle();
                    },

                    setMouseOverStyle : function() {
                        backgroundImage.setMouseOverStyle();
                    },

                    resetStyle : function() {
                        sourceUIDiv.css({
                            "border-color": colors.default.border,
                            "background-color": colors.default.background
                        });
                        backgroundImage.setDefaultStyle();
                    }
                }
            }
        }
    }

    function OffsetService( context ){

        function getRelativeStageOffset( element ){

            var jqElement = jq(element),
                elementOffset = jqElement.offset(),
                stageOffset = context.stage.getOffset(),
                stageZoom = context.stage.getZoom();

            return {
                left : (elementOffset.left - stageOffset.left) / stageZoom,
                top : (elementOffset.top - stageOffset.top) / stageZoom
            };
        }

        function removeGroupOffset( offset, element ){

            var jqElement = jq(element),
                offsetCorrection = getGroupOffsetCorrection( jqElement ),
                stageZoom = context.stage.getZoom();

            return {
                left : offset.left + (offsetCorrection.left / stageZoom),
                top : offset.top + (offsetCorrection.top / stageZoom)
            };
        }

        function getGroupOffsetCorrection( jqElement ) {

            var offsetCorrection = {
                    left: 0,
                    top: 0
                },
                parents = jqElement.parents(),
                parent,
                position;


            parents.each( function( index, element ){
                parent = jq(element);
                if( parent.hasClass("Group") || parent.hasClass("ttclassgroup") ){
                    position = parent.position();
                    offsetCorrection.left -= position.left;
                    offsetCorrection.top -= position.top;
                }
            });

            return offsetCorrection;
        }

        function getPointerPositionFromEvent( event ){

            if( event.originalEvent ) {

                if( event.originalEvent.touches && event.originalEvent.touches[0] ){
                    return event.originalEvent.touches[0];
                }

                if( event.originalEvent.changedTouches && event.originalEvent.changedTouches[0] ){
                    return event.originalEvent.changedTouches[0];
                }
            }

            return event;
        }

        return {
            getRelativeStageOffset : getRelativeStageOffset,
            removeGroupOffset : removeGroupOffset,
            getPointerPositionFromEvent : getPointerPositionFromEvent
        }
    }

    function DropSourceStore( pageId, scId, context ){

        var variableNamespace = 'com.tts.dragAndDrop',
            variableId = "DragAndDropSource_" + pageId + "_" + scId;

        return {
            clear : function(){
                context.stage.variable.set(variableId + "_evaluated", null, variableNamespace);
                context.stage.variable.set(variableId + "_quiz_state", null, variableNamespace);
                context.stage.variable.set( variableId + "_drop_status", null, variableNamespace);
                context.stage.variable.set('DragAndDropTargetRegistry', null, variableNamespace );
            },
            getTargetRegistry : function(){
                return context.stage.variable.get('DragAndDropTargetRegistry', variableNamespace );
            },
            isEvaluated: function() {
                return context.stage.variable.get(variableId + "_evaluated", variableNamespace) === true;
            },
            setEvaluated: function() {
                context.stage.variable.set(variableId + "_evaluated", true, variableNamespace);
            },
            getQuizState: function() {
                return context.stage.variable.get(variableId + "_quiz_state", variableNamespace);
            },
            setQuizState: function( quizState ) {
                context.stage.variable.set(variableId + "_quiz_state", quizState, variableNamespace);
            },
            setDropStatus : function( targetId, targetIndex, droppedCorrect ){
                context.stage.variable.set( variableId + "_drop_status", {
                    droppedOnTarget : targetId,
                    targetIndex : targetIndex,
                    droppedCorrect : droppedCorrect
                }, variableNamespace );
            },
            getDropStatus: function() {
                var droppedStatus = context.stage.variable.get(variableId + "_drop_status", variableNamespace);
                return (droppedStatus === null) ? null : droppedStatus;
            }
        };
    }

    function SourceState( dropSourceStore ) {

        var currentTargetId = null,
            targetIndex = -1,
            droppedCorrectly = false;

        function hasValidTargetId(){
            return currentTargetId && currentTargetId !== "";
        }

        return {

            reset : function( sourceHasTarget ) {
                currentTargetId = null;
                droppedCorrectly = !sourceHasTarget;
            },

            restore : function() {

                var sourceInfo = dropSourceStore.getDropStatus();
                if( sourceInfo === null ) {
                    return false;
                }

                currentTargetId = sourceInfo.droppedOnTarget;
                targetIndex = sourceInfo.targetIndex;
                droppedCorrectly = sourceInfo.droppedCorrect;

                return true;
            },

            store : function() {
                dropSourceStore.setDropStatus( currentTargetId, targetIndex, droppedCorrectly );
            },

            droppedOnCorrectTarget : function( target ){
                droppedCorrectly = true;
                currentTargetId = target.getTargetId();
            },

            droppedOnWrongTarget : function( target ){
                droppedCorrectly = false;
                currentTargetId = target ? target.getTargetId() : null;
            },

            droppedOnInvalidTarget : function( sourceHasTarget ){
                droppedCorrectly = !sourceHasTarget;
                currentTargetId = null;
            },

            isDroppedCorrectly : function(){
                return droppedCorrectly;
            },

            hasValidTargetId : hasValidTargetId,
            getTargetId : function(){
                return hasValidTargetId() ? currentTargetId : null;
            },

            setTargetIndex : function( index ){
                targetIndex = index;
            },
            getTargetIndex : function(){
                return targetIndex;
            }
        }
    }

    function TargetRegistry( dropSourceStore, context, metadata ) {

        var targets = null;

        function getTargetForCurrentPosition( position ) {

            var overTarget = null;
            jq.each( targets, function( index, target ){
                if( isValidTarget( target ) && isPositionOnTarget( target.getPosition(), position ) ){
                    overTarget = target;
                    return false; // jquery.each break
                }

                return true; // jquery.each continue
            });

            return overTarget;
        }

        function isPositionOnTarget( targetPosition, position ) {

            var offset = targetPosition.offset,
                stageZoom = context.stage.getZoom();

            if( !offset ) {
                return false;
            }

            if( position.x < offset.left || position.x > (offset.left + (targetPosition.size.width * stageZoom)) ) {
                return false;
            }

            if( position.y < offset.top || position.y > (offset.top + (targetPosition.size.height * stageZoom)) ) {
                return false;
            }

            return true;
        }


        function isValidTarget(target) {

            if( !target ){
                return false;
            }

            if( typeof(target.getPosition) !== "function" ){
                return false;
            }

            if( typeof(target.getTargetId) !== "function" ){
                return false;
            }

            if( typeof(target.onDragStart) !== "function" ){
                return false;
            }

            if( typeof(target.onDragOver) !== "function" ){
                return false;
            }

            if( typeof(target.onDragOut) !== "function" ){
                return false;
            }

            if( typeof(target.onDrop) !== "function" ){
                return false;
            }

            if( typeof(target.setAsBestPracticeSolution) !== "function" ){
                return false;
            }

            return true;
        }

        function getTargetById( targetId ){

            var targetComponent = null,
                currentTargetId;

            jq.each( targets, function( index, target ) {
                currentTargetId = target.getTargetId();
                if(  currentTargetId === targetId ){
                    targetComponent = target;
                    return false; // jquery.each break
                }

                return true; // jquery.each continue
            });

            return targetComponent;
        }

        return {

            initialize : function() {

                var stepId = metadata.page.getId(),
                    targetRegistry = dropSourceStore.getTargetRegistry();

                if( targetRegistry === null ){
                    return;
                }

                targets = targetRegistry[stepId] || [];
            },

            getTargetByPosition : getTargetForCurrentPosition,
            getTargetById : getTargetById
        }
    }

    function QuizState( dropSourceStore ) {

        var states = {
                Undefined : "undefined",
                TestRun: "test_run",
                UserSolution: "user_solution",
                BestPracticeSolution: "best_practice_solution"
            },
            currentState = states.Undefined,
            previousState = states.Undefined;

        function setState( newState ){
            previousState = currentState;
            currentState = newState;
            dropSourceStore.setQuizState( currentState );
        }

        return {
            changeToTestRun : function(){
                setState( states.TestRun );
            },
            wasChangingFromTestRun : function(){
                return previousState === states.TestRun;
            },
            changeToUserSolution : function(){
                setState(  states.UserSolution );
            },
            changeToBestPracticeSolution : function(){
                setState( states.BestPracticeSolution );
            },
            restore : function(){
                currentState = dropSourceStore.getQuizState();
            }
        };
    }

    /**
     * @constructor
     */
    return function ( context, metadata ) {

        // declare private members
        var self = null,
            parameters = metadata.parameters,
            isStandardPlayer = context.getPlayerName() === "standard",

            offsetService = new OffsetService( context ),
            dropSourceStore = new DropSourceStore( metadata.page.getId(), metadata.getId(), context ),
            targetRegistry = new TargetRegistry( dropSourceStore, context, metadata ),
            sourceState = new SourceState( dropSourceStore ),
            quizState = new QuizState( dropSourceStore ),
            sourceObject,
            destinationTargetIds = getDestinationTargetIds( "target_", 20 ),
            pointerUpEventName,
            pointerDownEventName,
            pointerMoveEventName,

            dragOverTarget = null,
            dropTarget = null,

            dragging = false,

            mousePosition = {
                "x": -1,
                "y": -1 },
            mouseDownPosition = {
                "x" : 0,
                "y" : 0
            },
            zoomCorrection = {
                "top" : 0,
                "left" : 0
            };

        function initPointerEventNames() {
            if( !pointerUpEventName ) {
                pointerUpEventName = EventUtil.getPointerUpEvent( sourceObject.getDragDiv() );
                pointerDownEventName = EventUtil.getPointerDownEvent( sourceObject.getDragDiv() );
                pointerMoveEventName = EventUtil.getPointerMoveEvent( sourceObject.getDragDiv() );
            }
        }

        function getDestinationTargetIds( id, max ) {

            var result = [],
                i;

            for( i=1; i <= max; i++ ) {
                if (parameters.get(id + i) === "true") {
                    result.push( i.toString() );
                }
            }

            return result;
        }

        function createComponent() {
            self = this;
            setComponentBehaviour();
            sourceObject = new SourceElementBuilder( context, metadata, offsetService ).build();
        }

        function setComponentBehaviour() {
            context.page.quiz.add( self, "1", true );
            context.page.event.bindOnEnter( self );
            context.page.event.bindOnExit( self );
        }

        function resetStates() {

            dragOverTarget = null;
            dropTarget = null;
            dragging = false;

            sourceObject.initialize();
            sourceState.reset( hasDestinationTarget() );

            if( context.stage.isInPreviewMode() ){
                dropSourceStore.clear();
                targetRegistry.initialize();
            }
        }

        function hasDestinationTarget() {
            return destinationTargetIds.length > 0;
        }

        function isEvaluated(){
            return dropSourceStore.isEvaluated();
        }

        function moveToStartPosition( skipAnimation ){
            sourceObject.moveToStartOffset( skipAnimation );
        }

        function moveToTargetPosition( offset, index ) {

            if( !isEvaluated() ) {
                sourceState.setTargetIndex( index );
                sourceState.store();
            }

            sourceObject.moveToOffset( offset );
        }

        function bindEvents() {

            if (context.stage.isInAuthoringMode()) {
                return;
            }

            removeEvents();

            EventUtil.attachFirstSupportedEvent( sourceObject.getDragDiv(), pointerDownEventName, dragStart );
            EventUtil.attachFirstSupportedEvent( sourceObject.getDragDiv(), pointerMoveEventName, preventIOS8VerticalMove );
            EventUtil.attachFirstSupportedEvent( sourceObject.getDragDiv(), pointerUpEventName, drop );

            jq(document)
                .on( pointerDownEventName + " " + pointerMoveEventName, getGlobalMousePosition )
                .on( pointerUpEventName, drop );
        }

        function getGlobalMousePosition (event) {

            var positionData = offsetService.getPointerPositionFromEvent( event );
            mousePosition.x = positionData.pageX;
            mousePosition.y = positionData.pageY;

            if( dragging ) {
                drag();
            }
        }

        function removeEvents() {

            initPointerEventNames();

            EventUtil.removeFirstSupportedEvent( sourceObject.getDragDiv(), pointerDownEventName, dragStart );
            EventUtil.removeFirstSupportedEvent( sourceObject.getDragDiv(), pointerMoveEventName, preventIOS8VerticalMove );
            EventUtil.removeFirstSupportedEvent( sourceObject.getDragDiv(), pointerUpEventName, drop );

            jq(document)
                .off( pointerDownEventName + " " + pointerMoveEventName, getGlobalMousePosition )
                .off( pointerUpEventName, drop );
        }

        function preventIOS8VerticalMove(e) {
            e.preventDefault ? e.preventDefault() : true;
        }

        function dragStart() {

            if( sourceObject.isInAnimation() ){
                return;
            }

            lockNavigation();

            dragging = true;

            sourceObject.setDragStyle();

            var stageZoom = context.stage.getZoom(),
                offset = sourceObject.getDragDiv().offset();

            zoomCorrection.top = (metadata.getHeight()/2) * stageZoom;
            zoomCorrection.left = (metadata.getWidth()/2) * stageZoom;

            mouseDownPosition.x = mousePosition.x - offset.left;
            mouseDownPosition.y = mousePosition.y - offset.top;

            checkAndNotifyTargetOnDragStart();
            checkAndNotifyTargetOnDrag();
        }

        function lockNavigation() {
            if( context.stage.trigger && typeof( context.stage.trigger.lock ) === "function" ) {
                context.stage.trigger.lock(metadata.getId());
            }
        }

        function drag() {

            var offset,
                isTouchDevice = ('ontouchstart' in window) || (navigator.MaxTouchPoints > 0) || (navigator.msMaxTouchPoints > 0) || context.device.isMobile();

            if( isTouchDevice ){
                offset = {
                    top: mousePosition.y - zoomCorrection.top,
                    left: mousePosition.x - zoomCorrection.left
                };
            }
            else {
                offset = {
                    top: (mousePosition.y - mouseDownPosition.y),
                    left: (mousePosition.x - mouseDownPosition.x)
                };
            }

            sourceObject.setOffset( offset );
            sourceObject.clearCursorStyle();

            checkAndNotifyTargetOnDrag();
        }

        function checkAndNotifyTargetOnDragStart() {
            if( dropTarget ){
                dropTarget.onDragStart( self );
                dropTarget = null;
            }
        }

        function checkAndNotifyTargetOnDrag() {

            var currentDragOverTarget = targetRegistry.getTargetByPosition( mousePosition );
            if( dragOverTarget && dragOverTarget !== currentDragOverTarget ) {
                dragOverTarget.onDragOut( self );
                dragOverTarget = null;
            }

            if (currentDragOverTarget && currentDragOverTarget !== dragOverTarget) {
                currentDragOverTarget.onDragOver(self);
                dragOverTarget = currentDragOverTarget;
            }
        }

        function drop() {

            if( !dragging ) {
                return;
            }

            dragging = false;
            unlockNavigation();
            checkAndNotifyTargetOnDrop();
            sourceObject.setDroppedStyle( sourceState.hasValidTargetId() );
        }

        function unlockNavigation() {
            if( context.stage.trigger && typeof( context.stage.trigger.unlock ) === "function" ) {
                context.stage.trigger.unlock(metadata.getId());
            }
        }

        function checkAndNotifyTargetOnDrop() {

            var target = targetRegistry.getTargetByPosition( mousePosition );
            if( target && target.onDrop( self )) {

                if( isCorrectAssignment( target ) ){
                    sourceState.droppedOnCorrectTarget( target );
                    context.page.quiz.setSelected( self, true );
                }
                else {
                    sourceState.droppedOnWrongTarget( target );
                    context.page.quiz.setSelected( self, false );
                }

                dropTarget = target;
                dragOverTarget = null;
            }
            else {
                sourceState.droppedOnInvalidTarget( hasDestinationTarget() );
                moveToStartPosition();
            }

            sourceState.store();
        }

        function onUserSolution() {

            quizState.changeToUserSolution();
            removeEvents();
            setEvaluated();
            sourceObject.setUserSolutionStyle( sourceState.isDroppedCorrectly() );

            if( quizState.wasChangingFromTestRun() ) {
                return;
            }

            // use timeout to let the target react to the async event "OnUserSolution" and to restore the users order.
            setTimeout( reassignSourceToUserSelectedTarget, getTimeoutByTargetIndex() );
        }

        function setEvaluated(){
            dropSourceStore.setEvaluated();
        }

        function getTimeoutByTargetIndex(){

            if( sourceState.getTargetIndex() >= 0 ){
                return (sourceState.getTargetIndex() + 1) * 10;
            }
            return 10;
        }

        function reassignSourceToUserSelectedTarget() {

            var target;

            if( sourceState.hasValidTargetId() ) {
                target = targetRegistry.getTargetById( sourceState.getTargetId() );
                if( target ) {
                    target.setAsUserSolution( self );
                }
                else {
                    console.error("Target with id " +  sourceState.getTargetId() + "not found!");
                }
                return;
            }

            moveToStartPosition();
        }

        function onBestPracticeSolution() {

            quizState.changeToBestPracticeSolution();
            removeEvents();
            sourceObject.setBestPracticeSolutionStyle();

            assignSourceToBestPracticeTarget();
        }

        function assignSourceToBestPracticeTarget() {

            // use timeouts to let the target react to the async event "OnBestPracticeSolution"
            // and place the targets in the right order

            if( placeSourceOnUserAssignedTarget() ) {
                return;
            }

            // use timeout to place not correct assigned source after the correct assigned
            setTimeout( placeSourceOnFirstAvailableTarget, getTimeoutByTargetIndex() + 100 );
        }

        function placeSourceOnUserAssignedTarget(){

            var target;
            if(sourceState.hasValidTargetId() && sourceState.isDroppedCorrectly() ) {
                target = targetRegistry.getTargetById( sourceState.getTargetId() );
                if( target ) {
                    setTimeout( function(){ target.setAsBestPracticeSolution(self); }, getTimeoutByTargetIndex() );
                }
                return true;
            }

            return false;
        }

        function placeSourceOnFirstAvailableTarget(){

            var placed = false,
                target;

            jq.each( destinationTargetIds, function( index, targetId ){

                target = targetRegistry.getTargetById( targetId  );
                if( target && isCorrectAssignment( target ) && target.setAsBestPracticeSolution( self ) ){
                    placed = true;
                    return false; // jquery.each break
                }

                return true; // jquery.each continue
            });

            if( !placed ){
                moveToStartPosition();
            }
        }

        function isCorrectAssignment( target ) {

            var correctAssigned = false;
            jq.each( destinationTargetIds, function( index, id ){
                if( id === target.getTargetId() ) {
                    correctAssigned = true;
                    return false; // jquery.each break
                }

                return true; // jquery.each continue
            });

            return correctAssigned;
        }

        function initializeTestRun() {
            sourceObject.resetStyle();
            bindEvents();
            quizState.changeToTestRun();
        }

        function initializeStandardPlayer(){
            initializeTestRun();
        }

        function cleanupStandardPlayer(){
            moveToStartPosition(true);
        }

        function initializeHTML5Player() {

            if (isEvaluated()) {
                sourceState.restore();
                quizState.restore();
                return;
            }

            if (context.page.quiz.hasQuestion()) {
                initializeTestRun();
            }
        }

        function cleanupHTML5Player(){
            if( !isEvaluated() ) {
                moveToStartPosition(true);
            }
        }

        return {
            // declare public members
            onLoading : createComponent,
            onEnterPage : resetStates,
            onAllComponentsReady : function(){
                targetRegistry.initialize();
                isStandardPlayer ? initializeStandardPlayer() : initializeHTML5Player();
            },
            onExitPage : function() {
                removeEvents();
                isStandardPlayer ? cleanupStandardPlayer() : cleanupHTML5Player();
                resetStates();
            },
            isCorrectAssignment : isCorrectAssignment,
            onPositionOnTargetChanged : moveToTargetPosition,
            onUserSolution : onUserSolution,
            onBestPracticeSolution : onBestPracticeSolution,
            getId : function() {
                return metadata.getId();
            },
            getHeight : function() {
                return metadata.getHeight();
            }
        };
    };
});
