define( function() {

    var fallbackLanguage = "en-us",
        texts = {
            "player": {
                "navigation": {
                    "modes": {
                        "interactive": {
                            "de-de": "Interaktiver Modus",
                            "en-us": "Interactive Mode",
                            "es-es": "Modo interactivo",
                            "fr-fr": "Mode interactif"
                        },
                        "presentation": {
                            "de-de": "Präsentationsmodus",
                            "en-us": "Presentation Mode",
                            "es-es": "Modo presentación",
                            "fr-fr": "Mode de présentation"
                        },
                        "film": {
                            "de-de": "Film-Modus",
                            "en-us": "Film Mode",
                            "es-es": "Modo de película",
                            "fr-fr": "Mode Film"
                        }
                    }
                }
            }
        };

    /**
     * Gets data from a nested object
     *
     * @private
     * @param {string} key - a dot separated representation of the objectstructure e.g.("cmi.core.lesson_status")
     * @param {object|string} data - the data object where the value should be got from
     */
    function getData( key, data ) {

        var parts = key.split("."),
            i = 0,
            parent = data;

        for(; i < parts.length; ++i) {
            if (!parent[parts[i]]) {
                return null;
            }
            parent = parent[parts[i]];
        }

        return parent;
    }

    // revealing public API
    return {

        /**
         * Gets a text in a given language
         * If the text is not defined in the given language, the text in teh fallback "en-us" is used, otherwise "null".
         *
         * @public
         * @param {string} key - a dot separated representation of the resource e.g.("quiz.evaluate")
         * @param {string} [language] - the language for the text e.g.("en-us")
         */
        get : function( key, language ){

            var text = getData( key, texts );
            if( text ){
                return getData( language, text ) || getData( fallbackLanguage, text );
            }

            return null;
        }
    }
});
