// com.tts.initial.html5.quiz.button.evaluation
// HTML4 / HTML5: No differences
define(["jquery"], function ( jq ) {

	"use strict";

    var EventUtil = (function() {

        var
            isIOS = (/iphone|ipad|ipod/i.test( navigator.userAgent.toLowerCase() ) );

        function getFirstSupportedEvent(tagName, eventNames) {
            var result = "";

            jq.each(eventNames.split(" "), function (i, evtName) {
                if (isEventSupported(evtName, tagName)) {
                    result = evtName;
                    return false;
                }
            });
            return result;
        }

        function isEventSupported(eventName, tagName) {
            var elem = tagName ? document.createElement(tagName) : document,
                onEventName = "on" + eventName,
                isSupported;

            isSupported = !!(onEventName in elem);
            if (!isSupported && typeof elem.setAttribute !== "undefined") {
                elem.setAttribute(onEventName, "return;");
                isSupported = typeof elem[onEventName] === "function";
            }

            return isSupported;
        }

        return {
            /**
             * Attach the event handler <b>handler</b> for the first supported event specified by <b>eventNames</b>
             * to the element <b>jqObject</b>.
             * Example: attachFirstSupportedEvent( myObject, "pointerdown mousedown touchend", myHandler );
             *
             * @param element
             * @param {String} eventNames
             * @param handler
             * @return {String} the event name or ""
             */
            attachFirstSupportedEvent: function (element, eventNames, handler) {
                var jqObject = jq(element),
                    evtName = getFirstSupportedEvent(element.nodeName, eventNames);

                if (!evtName) {
                    return "";
                }

                jqObject.on(evtName, handler);
                return evtName;
            },

            /**
             * Remove attached event handler <b>handler</b> for the first supported event specified by <b>eventNames</b>
             * from the element <b>jqObject</b>.
             * Example: removeFirstSupportedEvent( myObject, "pointerdown mousedown touchend", myHandler );
             *
             * @param element
             * @param {String} eventNames
             * @param handler
             * @return {String} the event name or ""
             */
            removeFirstSupportedEvent: function (element, eventNames, handler) {
                var jqObject = jq(element),
                    evtName = getFirstSupportedEvent(element.nodeName, eventNames);

                if (!evtName) {
                    return "";
                }

                jqObject.off(evtName, handler);
                return evtName;
            },

            getPointerDownEvent: function( element ) {
                if( !isIOS ) {
                    return getFirstSupportedEvent( element.nodeName, "pointerdown mousedown touchstart" );
                }

                // iOS 9.3 unterstützt zwar click und mouseup, weshalb sich auf diese Events attached wird, aber sie werden
                //  aus unbekannten Gründen nicht gesendet.
                return "touchstart";
            },

            getPointerUpEvent: function( element ) {
                if( !isIOS ) {
                    return getFirstSupportedEvent( element.nodeName, "click pointerup mouseup touchend" );
                }

                // iOS 9.3 unterstützt zwar click und mouseup, weshalb sich auf diese Events attached wird, aber sie werden
                //  aus unbekannten Gründen nicht gesendet.
                return "touchend";
            },

            getPointerMoveEvent: function( element ) {
                if( !isIOS ) {
                    return getFirstSupportedEvent( element.nodeName, "pointermove mousemove touchmove" );
                }

                return "touchmove";
            }
        };
    })();

    /**
	 * @constructor
	 */
	return function (context, metadata) {

		var uiText = (function() {

                var fallbackLanguage = "en-us",
                    texts = {
                        "player": {
                            "quiz": {
                                "button": {
                                    "submit": {
                                        "et-ee": "Esita vastus",
                                        "bg-bg": "Подаване на отговор",
                                        "cs-cz": "Zadat odpověď",
                                        "da-dk": "Afgiv svar",
                                        "de-de": "Antwort abgeben",
                                        "en-us": "Submit answer",
                                        "es-es": "Enviar respuesta",
                                        "fi-fi": "Anna vastaus",
                                        "fr-fr": "Soumettre la réponse",
                                        "el-gr": "Παρέχετε απάντηση",
                                        "hi-in": "उत्तर सबमिट करें",
                                        "hr-hr": "Odgovori",
                                        "hu-hu": "Válasz megadása",
                                        "it-it": "Invia risposta",
                                        "ja-jp": "回答を送信する",
                                        "ko-kr": "답변 제출",
                                        "lt-lt": "Pateikti atsakymą",
                                        "mk-mk": "Поднеси одговор",
                                        "mn-mn": "Хариулт илгээх",
                                        "ms-id": "Kirim jawaban",
                                        "nl-nl": "Antwoord versturen",
                                        "no-no": "Send inn svar",
                                        "pl-pl": "Prześlij odpowiedź",
                                        "pt-pt": "Responder",
                                        "ro-ro": "Transmitere răspuns",
                                        "ru-ru": "Отправить ответ",
                                        "sk-sk": "Odoslať odpoveď",
                                        "sl-si": "Oddaj odgovor",
                                        "sr-rs": "Odgovori",
                                        "sv-se": "Skicka svar",
                                        "th-th": "ส่งคำตอบ",
                                        "tr-tr": "Cevapla",
                                        "zh-tw": "送出答案",
                                        "zh-cn": "提交答案",
                                        "de-at": "Antwort abgeben",
                                        "de-ch": "Antwort abgeben",
                                        "eo-de": "Antwort abgeben",
                                        "nl-be": "Antwoord versturen",
                                        "en-ie": "Submit answer",
                                        "en-gb": "Submit answer",
                                        "eo-en": "Antwort abgeben",
                                        "fr-be": "Soumettre la réponse"
                                    },
                                    "submitExceeded": {
                                        "et-ee": "Vastus esitatud",
                                        "bg-bg": "Отговорът е подаден",
                                        "cs-cz": "Odpověď byla zadána",
                                        "da-dk": "Svar afgivet",
                                        "de-de": "Antwort abgegeben",
                                        "en-us": "Answer submitted",
                                        "es-es": "Respuesta enviada",
                                        "fi-fi": "Vastaus annettu",
                                        "fr-fr": "Réponse soumise",
                                        "el-gr": "Παρασχέθηκε απάντηση",
                                        "hi-in": "उत्तर सबमिट किया गया",
                                        "hr-hr": "Odgovoreno",
                                        "hu-hu": "Válasz megadva",
                                        "it-it": "Risposta inviata",
                                        "ja-jp": "回答を送信しました",
                                        "ko-kr": "답변 제출됨",
                                        "lt-lt": "Atsakymas pateiktas",
                                        "mk-mk": "Одговорот е поднесен",
                                        "mn-mn": "Хариултыг илгээлээ",
                                        "ms-id": "Jawaban terkirim",
                                        "nl-nl": "Antwoord verstuurd",
                                        "no-no": "Svar sendt",
                                        "pl-pl": "Odpowiedź przesłana",
                                        "pt-pt": "Resposta entregue",
                                        "ro-ro": "Răspuns transmis",
                                        "ru-ru": "Ответ отправлен",
                                        "sk-sk": "Odpoveď odoslaná",
                                        "sl-si": "Odgovor je oddan",
                                        "sr-rs": "Odgovoreno",
                                        "sv-se": "Svar skickat",
                                        "th-th": "ส่งคำตอบแล้ว",
                                        "tr-tr": "Cevaplandırıldı",
                                        "zh-tw": "答案已送出",
                                        "zh-cn": "答案已提交",
                                        "de-at": "Antwort abgegeben",
                                        "de-ch": "Antwort abgegeben",
                                        "eo-de": "Antwort abgegeben",
                                        "nl-be": "Antwoord verstuurd",
                                        "en-ie": "Answer submitted",
                                        "en-gb": "Answer submitted",
                                        "eo-en": "Antwort abgegeben",
                                        "fr-be": "Réponse soumise"
                                    },
                                    "evaluate": {
                                        "et-ee": "Hindamine",
                                        "bg-bg": "Оценяване",
                                        "cs-cz": "Vyhodnocení",
                                        "da-dk": "Analyse",
                                        "de-de": "Auswertung",
                                        "en-us": "Evaluate",
                                        "es-es": "Evaluación",
                                        "fi-fi": "Arviointi",
                                        "fr-fr": "Evaluation",
                                        "el-gr": "Αξιολόγηση",
                                        "hi-in": "मूल्यांकन",
                                        "hr-hr": "Vrednovanje",
                                        "hu-hu": "Kiértékelés",
                                        "it-it": "Valutazione",
                                        "ja-jp": "評価",
                                        "ko-kr": "평가",
                                        "lt-lt": "Vertinimas",
                                        "mk-mk": "Испит",
                                        "mn-mn": "Үнэлгээ",
                                        "ms-id": "Evaluasi",
                                        "nl-nl": "Oplossing",
                                        "no-no": "Vurdering",
                                        "pl-pl": "Ocena",
                                        "pt-pt": "Avaliação",
                                        "ro-ro": "Evaluare",
                                        "ru-ru": "Подведение итогов",
                                        "sk-sk": "Vyhodnotenie",
                                        "sl-si": "Obdelava",
                                        "sr-rs": "Ocena",
                                        "sv-se": "Bedömning",
                                        "th-th": "การประเมิน",
                                        "tr-tr": "Deðerlendirme",
                                        "zh-tw": "評估",
                                        "zh-cn": "评估",
                                        "ca-es": "Valoració",
                                        "eu-es": "Balorazioa",
                                        "es-ar": "Evaluación",
                                        "pt-br": "Avaliação",
                                        "de-at": "Auswertung",
                                        "de-ch": "Auswertung",
                                        "eo-de": "Auswertung",
                                        "nl-be": "Oplossing",
                                        "en-ie": "Evaluate",
                                        "en-gb": "Evaluate",
                                        "eo-en": "Auswertung",
                                        "fr-be": "Evaluation"
                                    },
                                    "bestPracticeSolution": {
                                        "et-ee": "Kuva lahendus",
                                        "bg-bg": "Показване на решението",
                                        "cs-cz": "Zobrazit øešení",
                                        "da-dk": "Vis løsning",
                                        "de-de": "zur Musterlösung",
                                        "en-us": "Show sample solution",
                                        "es-es": "Ver Solución correcta",
                                        "fi-fi": "Ratkaisun näyttäminen",
                                        "fr-fr": "Afficher la bonne solution",
                                        "el-gr": "Εμφάνιση λύσης",
                                        "hi-in": "समाधान प्रदर्शित करें",
                                        "hr-hr": "Prikazati riješenje",
                                        "hu-hu": "Megoldás bemutatása",
                                        "it-it": "Mostra soluzione modello",
                                        "ja-jp": "正解を表示",
                                        "ko-kr": "해답 표시",
                                        "lt-lt": "Rodyti sprendimą",
                                        "mk-mk": "Прикажи решение",
                                        "mn-mn": "Дэлгэцийн шийдэл",
                                        "ms-id": "Tampilkan jawaban",
                                        "nl-nl": "Oplossing tonen",
                                        "no-no": "Vis løsning",
                                        "pl-pl": "Pokaż rozwiązanie",
                                        "pt-pt": "Indicar solução",
                                        "ro-ro": "Afişarea soluţiei",
                                        "ru-ru": "Показать решение",
                                        "sk-sk": "Zobrazenie riešenie",
                                        "sl-si": "Pokaži rešitev",
                                        "sr-rs": "Prikaz rešenja",
                                        "sv-se": "Visa lösning",
                                        "th-th": "แสดงวิธีแก้ไขปัญหา",
                                        "tr-tr": "Çözümü göster",
                                        "zh-tw": "答複通知",
                                        "zh-cn": "答复通知",
                                        "ca-es": "Mostrar solució",
                                        "eu-es": "Erantzun zuzena ikusi",
                                        "es-ar": "Visualizar solución",
                                        "pt-br": "Indicar solução",
                                        "de-at": "zur Musterlösung",
                                        "de-ch": "zur Musterlösung",
                                        "eo-de": "zur Musterlösung",
                                        "nl-be": "Oplossing tonen",
                                        "en-ie": "Show sample solution",
                                        "en-gb": "Show sample solution",
                                        "eo-en": "zur Musterlösung",
                                        "fr-be": "Afficher la bonne solution"
                                    },
                                    "userSolution": {
                                        "et-ee": "Kuva testiküsimuse hinne",
                                        "bg-bg": "Оценяване",
                                        "cs-cz": "Vyhodnocení",
                                        "da-dk": "Analyse",
                                        "de-de": "Zurück zur Auswertung",
                                        "en-us": "Back to evaluation",
                                        "es-es": "Volver a la evaluación",
                                        "fi-fi": "Arviointi",
                                        "fr-fr": "Retour à l'evaluation",
                                        "el-gr": "Αξιολόγηση",
                                        "hi-in": "परीक्षा प्रश्न का मूल्यांकन प्रदर्शित करें",
                                        "hr-hr": "Vrednovanje",
                                        "hu-hu": "Kiértékelés",
                                        "it-it": "Indietro alla valutazione",
                                        "ja-jp": "テスト質問の評価を表示",
                                        "ko-kr": "시험 문제의 평가 표시",
                                        "lt-lt": "Vertinimas",
                                        "mk-mk": "Испит",
                                        "mn-mn": "Үнэлгээ",
                                        "ms-id": "Evaluasi",
                                        "nl-nl": "Oplossing",
                                        "no-no": "Vurdering",
                                        "pl-pl": "Ocena",
                                        "pt-pt": "Avaliação",
                                        "ro-ro": "Evaluare",
                                        "ru-ru": "Подведение итогов",
                                        "sk-sk": "Vyhodnotenie",
                                        "sl-si": "Obdelava",
                                        "sr-rs": "Ocena",
                                        "sv-se": "Visa bedömning av testfråga",
                                        "th-th": "แสดงการประเมินคำถามทดสอบ",
                                        "tr-tr": "Deðerlendirme",
                                        "zh-tw": "測試問題評估通知",
                                        "zh-cn": "测试问题评估通知",
                                        "ca-es": "Valoració",
                                        "eu-es": "Galderen balorazioa ikusi",
                                        "es-ar": "Volver a la evaluación",
                                        "pt-br": "Avaliação",
                                        "de-at": "Zurück zur Auswertung",
                                        "de-ch": "Zurück zur Auswertung",
                                        "eo-de": "Zurück zur Auswertung",
                                        "nl-be": "Oplossing",
                                        "en-ie": "Back to evaluation",
                                        "en-gb": "Back to evaluation",
                                        "eo-en": "Zurück zur Auswertung",
                                        "fr-be": "Retour à l'evaluation"
                                    }
                                }
                            }
                        }
                    };

                /**
                 * Gets data from a nested object
                 *
                 * @private
                 * @param {string} key - a dot separated representation of the objectstructure e.g.("cmi.core.lesson_status")
                 * @param {object|string} data - the data object where the value should be got from
                 */
                function getData( key, data ) {

                    var parts = key.split("."),
                        i = 0,
                        parent = data;

                    for(; i < parts.length; ++i) {
                        if (!parent[parts[i]]) {
                            return null;
                        }
                        parent = parent[parts[i]];
                    }

                    return parent;
                }

                // revealing public API
                return {

                    /**
                     * Gets a text in a given language
                     * If the text is not defined in the given language, the text in teh fallback "en-us" is used, otherwise "null".
                     *
                     * @public
                     * @param {string} key - a dot separated representation of the resource e.g.("quiz.evaluate")
                     * @param {string} [language] - the language for the text e.g.("en-us")
                     */
                    get : function( key, language ){

                        var text = getData( key, texts );
                        if( text ){
                            return getData( language, text ) || getData( fallbackLanguage, text );
                        }

                        return null;
                    }
                }
            })(),
            isStandardPlayer = context.getPlayerName() === "standard",
			contentLanguage = metadata.stage.getContentLanguage(),
			instance,
			evaluateButton,
			buttonHotspot,

            ButtonMode = {
                EVALUATE: "EVALUATE",
                SUBMIT: "SUBMIT",
                GOTO_USER_SOLUTION: "GOTO_USER_SOLUTION",
                GOTO_BEST_PRACTICE_SOLUTION: "GOTO_BEST_PRACTICE_SOLUTION",
                INACTIVE: "INACTIVE"
            },

			currentMode = ButtonMode.EVALUATE,

            ButtonState = {
                Default : "btn_state_default",
                Hover : "btn_state_hover",
                Pressed : "btn_state_pressed",
                Inactive : "btn_state_inactive"
            },

            currentState = ButtonState.Inactive,

			sPageEvaluatedId = metadata.page.getId() + "_evaluated",
			sElementVariableNameSpace = "com_tts_quiz",

			canFireTouchstart = "ontouchstart" in document,
			canFireTouchmove = "ontouchmove" in document,

            pointerDownEventName,
			pointerMoveEventName,
            pointerUpEventName,

            // in case that the display_text parameter does not exist, display the text
            // the parameter was removed from the component for a limited time span
            displayText = !metadata.parameters.get("display_text") ? true : metadata.parameters.asBool("display_text"),

         	isInAuthoringMode = context.stage.isInAuthoringMode(),
			isAssessmentMode = context.stage.isInAssessmentMode(),
		    hasStageTrigger = !!context.stage.trigger,
            background_position = {},

			 BUTTON_TEXT = {
				 EVALUATE: getText("textEvaluate","player.quiz.button.evaluate"),
                 SUBMIT: getText("textSubmit","player.quiz.button.submit"),
                 GOTO_BEST_PRACTICE_SOLUTION: getText("textGoToSampleSolution","player.quiz.button.bestPracticeSolution"),
				 GOTO_USER_SOLUTION: getText("textBackToEvaluation","player.quiz.button.userSolution")
			 },
			 BUTTON_TEXT_COLOR = {
				 EVALUATE: metadata.parameters.get("colorEvaluate",""),
                 SUBMIT: metadata.parameters.get("colorSubmit",""),
                 GOTO_BEST_PRACTICE_SOLUTION: metadata.parameters.get("colorGoToSampleSolution",""),
				 GOTO_USER_SOLUTION: metadata.parameters.get("colorBackToEvaluation",""),
				 INACTIVE: metadata.parameters.get("colorInactive","")
			 };

        function getText( parameterName, defaultTextName ) {
            var paramValue = jq.trim( metadata.parameters.get(parameterName,"") );
			return ( paramValue.length > 0 ) ? paramValue : uiText.get( defaultTextName, contentLanguage );
        }

        function getFontFamilyString(fontFamily) {
            switch (parseInt(fontFamily)) {
                case 2:
                    return "'Times New Roman', Times, serif";
                case 3:
                    return "Verdana, Geneva, sans-serif";
                case 4:
                    return "'Courier New', Courier, monospace";
                case 1:
                default:
            }
            return "Arial, Helvetica, sans-serif";
        }

        function hitTest(e) {
			var touch,
				mouseX = e.pageX,
				mouseY = e.pageY,
				jqElement = jq(e.target),
				offset = jqElement.offset(),
				x1 = offset.left,
				x2 = x1 + jqElement.width(),
				y1 = offset.top,
				y2 = y1 + jqElement.height();

			if (e.originalEvent && e.originalEvent.touches) {
				touch = e.originalEvent.touches[0] || e.originalEvent.changedTouches[0];
				mouseX = touch.pageX;
				mouseY = touch.pageY;
			}

			return mouseX >= x1 && mouseX <= x2 && mouseY >= y1 && mouseY <= y2;
		}

        function setupShellElement() {

            var shellElement = jq(context.getElement()),
				backgroundImage = metadata.parameters.get("bg_image", null),
                backgroundFileName,
                bg_width = 200,
                bg_height = 50;

            if (backgroundImage) {
                backgroundFileName = context.stage.getSharedPath() + backgroundImage.fileName;
                bg_width = backgroundImage.width / 3;
                bg_height = backgroundImage.height / 4;
            } else {
                backgroundFileName = context.getPath() + "evaluate_button.png";
            }

            initBackgroundPositions(bg_width, bg_height);

            shellElement.css({
                "width": bg_width + "px",
                "height": bg_height + "px"
            });

            return {
                width : bg_width,
				height : bg_height,
				filename : backgroundFileName
            };
        }

        function createButton( backgroundImage ) {

			var shellElement = jq(context.getElement()),
				button = jq(document.createElement("div"))
				.attr({
					"id": "evaluateQuizButton",
					"class": "quiz-button"
				})
				.css({
					"width": backgroundImage.width + "px",
					"height": backgroundImage.height + "px",
					"font-family": getFontFamilyString(metadata.parameters.asNumber("font-family",1)),
					"font-size": metadata.parameters.asNumber("font-size",14) + "px",
					"text-align": metadata.parameters.get("text_orientation","center"),
					"box-sizing": "border-box",
					"paddingTop": metadata.parameters.get("padding-top","15") + "px",
					"paddingRight": metadata.parameters.get("padding-right","15") + "px",
					"paddingLeft": metadata.parameters.get("padding-left","15") + "px",
					"color": getButtonColor()
				})
				.html( getButtonText() );

			if (metadata.parameters.asBool("display_background") === true) {
				button.css({
					"background-image": "url(\"" + backgroundImage.filename + "\")",
					"background-position": isAssessmentMode
                        ? getButtonImagePosition( ButtonMode.SUBMIT, ButtonState.Default )
                        : getButtonImagePosition( ButtonMode.EVALUATE, ButtonState.Default )
				});
			}

			shellElement.append(button);

			if (isInAuthoringMode) {
				createAuthoringShield(shellElement);
				return;
			}

            createButtonHotspot({
                width: backgroundImage.width,
                height: backgroundImage.height
			});

			return {
                getRootElement : function(){
                    return button;
                },
                getHotspotElement : function(){
                    return buttonHotspot;
                },
                hasState : function( buttonState ){
                    return buttonState === currentState;
                },
                changeState : function( buttonState ){

                    if( this.hasState( buttonState ) ){
                        return;
                    }

                    var enabled = buttonState !== ButtonMode.Inactive;

                    button.removeClass( currentState );
                    currentState = buttonState;
                    button.addClass( buttonState );
                    button.css({
                        "background-position" : getButtonImagePosition( currentMode, buttonState ),
                        "cursor": enabled ? "pointer" : "default"
                    });
                },
                changeMode: function( buttonMode ) {
                    if( currentMode === buttonMode ){
                        return;
                    }

                    button.removeClass( currentMode );
                    currentMode = buttonMode;
                    button.addClass( buttonMode );
                    button.css({
                        "color" : getButtonColor()
                    });
                }
            };
		}

		function getButtonText(){
        	if( !displayText || typeof(BUTTON_TEXT[currentMode]) === "undefined"){
        		return "";
			}

			return BUTTON_TEXT[currentMode];
		}

        function getButtonColor(){
            if( !displayText || typeof(BUTTON_TEXT_COLOR[currentMode]) === "undefined"){
                return "";
            }

            return BUTTON_TEXT_COLOR[currentMode];
        }

        function createButtonHotspot( size ){

            var shellElement = jq(context.getElement());

            buttonHotspot = jq(document.createElement("div"))
                .attr({
                    "class": "quiz-button-hotspot"
                })
                .css({
                    "position": "absolute",
                    "left": "0",
                    "top": "0",
                    "background-color": "#ffffff",
                    "opacity": "0.0",
                    "width": size.width + "px",//"100%",
                    "height": size.height + "px",//"100%",
                    "z-index": "9999"
                });

            shellElement.append( buttonHotspot );
        }

        function lockNavigation() {
			if (isInAuthoringMode || !hasStageTrigger) {
				return;
			}
			context.stage.trigger.lock(metadata.getId());
		}

		function unlockNavigation() {
			if (isInAuthoringMode || !hasStageTrigger) {
				return;
			}
			context.stage.trigger.unlock(metadata.getId());
		}

		function preventDefault(e) {
			e.preventDefault ? e.preventDefault() : true;
		}

		function initBackgroundPositions( tileImageWidth, tileImageHeight ) {

        	var buttonOrder = [
					"EVALUATE",
					"GOTO_BEST_PRACTICE_SOLUTION",
					"GOTO_USER_SOLUTION",
					"SUBMIT"
				],
                buttonPosX = -1 * tileImageWidth,
                buttonPosY = - 1 * tileImageHeight,
                currentButtonName,
                stateIndex;

        	background_position = {};
            jq.each( buttonOrder, function( orderIndex, buttonName ){
                stateIndex = 0;
                jq.each( ButtonState, function( key, buttonState ){
                    currentButtonName = buttonName + "_" + buttonState;
                    background_position[ currentButtonName.toUpperCase() ] = (orderIndex * buttonPosX) + "px " + (stateIndex * buttonPosY) + "px";
                    stateIndex++;
                });
            });
		}

		// create an object shield so we are able to select and drag element
		function createAuthoringShield(shellElement) {
			var objShield = jq(document.createElement("div"))
				.css({
					"position": "absolute",
					"left": "0",
					"top": "0",
					"background-color": "#ffffff",
					"opacity": "0.4",
					"width": "100%",
					"height": "100%",
					"z-index": "9999"
				});

			if (metadata.parameters.asBool("display_background") === false ) {
				objShield.css({
					"background-image": "url(\"" + context.getPath() + "invis.png\")"
				});
			}
			shellElement.append(objShield);
		}


		function getButtonImagePosition( buttonMode, state ){
        	return background_position[ buttonMode + "_" + state.toUpperCase() ];
		}

        function getHotspotElementNodeName() {
            return evaluateButton.getHotspotElement().get()
                ? evaluateButton.getHotspotElement().get(0).nodeName
                : "document";
        }

        function initPointerEventNames() {
            if( pointerDownEventName ) {
                return;
            }

            var nodeName = getHotspotElementNodeName();
            pointerDownEventName = EventUtil.getPointerDownEvent( nodeName );
            pointerUpEventName = EventUtil.getPointerUpEvent( nodeName );
            pointerMoveEventName = EventUtil.getPointerMoveEvent( nodeName );
        }

        function onMouseLeave() {
            unlockNavigation();

            if( isEnabled() ) {
                evaluateButton.changeState( ButtonState.Default );
            }
        }

        function onMouseOver() {
            if( isEnabled() && !evaluateButton.hasState( ButtonState.Pressed ) ) {
                evaluateButton.changeState( ButtonState.Hover );
            }
        }

        function onMouseDown() {
            lockNavigation();

            if( isEnabled() ) {
                evaluateButton.changeState( ButtonState.Pressed );
            }

            return true;
        }

        function onMouseUp(e) {
            unlockNavigation();

            if( !isEnabled() ) {
                return;
            }

            evaluateButton.changeState( ButtonState.Hover );

            if( !hitTest(e) ) {
                return;
            }

            if( currentMode === ButtonMode.EVALUATE || currentMode === ButtonMode.SUBMIT ) {
                context.page.quiz.evaluate();
            }
            else if( currentMode === ButtonMode.GOTO_BEST_PRACTICE_SOLUTION ){
                context.page.quiz.switchToBestPracticeSolution();
            }
            else if( currentMode === ButtonMode.GOTO_USER_SOLUTION ){
                context.page.quiz.switchToUserSolution();
            }

            return true;
        }

		function bindEvents() {

            if( isInAuthoringMode || !buttonHotspot ) {
				return;
			}

            removeEvents();
            EventUtil.attachFirstSupportedEvent( evaluateButton.getHotspotElement(), pointerDownEventName, onMouseDown );
            EventUtil.attachFirstSupportedEvent( evaluateButton.getHotspotElement(), pointerMoveEventName, onMouseOver );
            EventUtil.attachFirstSupportedEvent( evaluateButton.getHotspotElement(), "pointerout touchout mouseout", onMouseLeave );
            EventUtil.attachFirstSupportedEvent( evaluateButton.getHotspotElement(), pointerUpEventName, onMouseUp );

            if( canFireTouchmove ) {
                buttonHotspot.on("touchmove", preventDefault);
			}
		}

		function removeEvents() {

			if(!buttonHotspot) {
				return;
			}

            initPointerEventNames();

            EventUtil.removeFirstSupportedEvent( evaluateButton.getHotspotElement(), pointerDownEventName, onMouseDown );
            EventUtil.removeFirstSupportedEvent( evaluateButton.getHotspotElement(), pointerMoveEventName, onMouseOver );
            EventUtil.removeFirstSupportedEvent( evaluateButton.getHotspotElement(), "pointerout touchout mouseout", onMouseLeave );
            EventUtil.removeFirstSupportedEvent( evaluateButton.getHotspotElement(), pointerUpEventName, onMouseUp );

			if(canFireTouchmove) {
                buttonHotspot.off("touchmove", preventDefault);
			}
		}

		function refreshButton() {
            evaluateButton.getRootElement()
                .html( getButtonText() )
                .show();
        }

		function showEvaluateButton() {
            enableButton( isAssessmentMode ? ButtonMode.SUBMIT : ButtonMode.EVALUATE );
		}

        function showGotoUserSolutionButton() {
            enableButton( ButtonMode.GOTO_USER_SOLUTION );
        }

        function showGotoBestPracticeSolutionButton() {
            enableButton( ButtonMode.GOTO_BEST_PRACTICE_SOLUTION );
        }

        function showSubmitExceededButton() {
            if( displayText ) {
                evaluateButton.getRootElement()
                    .html( getText("textSubmitExceeded", "player.quiz.button.submitExceeded") );
            }

			disableButton();
        }

        function hideButton() {
            evaluateButton.getRootElement().hide();
            removeEvents();
		}

		function isEnabled() {
            return currentState !== ButtonState.Inactive;
        }

		function enableButton( buttonMode ) {
            evaluateButton.changeState( ButtonState.Default );
            evaluateButton.changeMode( buttonMode );
            refreshButton();
			bindEvents();
		}

        function disableButton() {
            if( !isEnabled() ){
                return;
            }

            evaluateButton.changeState( ButtonState.Inactive );
            evaluateButton.changeMode( ButtonMode.INACTIVE );
            removeEvents();
        }

        function isEvaluated() {

			if (context.stage.isInPreviewMode()) {
				return false;
			}

			if (isStandardPlayer) {
				return context.stage.variable.get(sPageEvaluatedId, sElementVariableNameSpace);
			}

			return context.page.quiz.hasQuestion() && context.page.quiz.getQuestion().isFinalized();
		}

		function setMouseOverState(){
            if( !canFireTouchstart ) {
                evaluateButton.getRootElement()
                    .css("background-position", getButtonImagePosition(currentMode, "hover"));
            }
		}

		function switchToUserSolution() {

            // ToDo: turn feature back on once API method context.stage.navigation.logic.getNextPageAccess() is working
            // This feature has been disabled as long as the API method
            // context.stage.navigation.logic.getNextPageAccess()
            // is not yet working correctly. Next button, Evaluate button and Memory game can interact and block the
            // next button navigation. Without the mentioned method working, user is still able to navigate to next page
            // by using the arrow keys on the keyboard
            /*
            setAllowNavigationNext( instance, metadata.parameters.asBool("disableNextButton",false) );
            */

            context.page.event.trigger(instance, "switch_to_evaluation");

            if ( isAssessmentMode ) {
                showSubmitExceededButton();
                setMouseOverState();
            	return;
            }

            if (metadata.parameters.asNumber("enable_sample_solution", 1)) {
				showGotoBestPracticeSolutionButton();
                setMouseOverState();
				return;
			}

			disableButton();
		}

		function switchToBestPracticeSolution() {

            context.page.event.trigger(instance, "switch_to_sample_solution");

			if (metadata.parameters.asNumber("enable_sample_solution", 1)) {
				showGotoUserSolutionButton();
                setMouseOverState();
			}
			else {
				disableButton();
			}

			// ToDo: turn feature back on once API method context.stage.navigation.logic.getNextPageAccess() is working
			// This feature has been disabled as long as the API method
			// context.stage.navigation.logic.getNextPageAccess()
			// is not yet working correctly. Next button, Evaluate button and Memory game can interact and block the
			// next button navigation. Without the mentioned method working, user is still able to navigate to next page
			// by using the arrow keys on the keyboard
			/*
			setAllowNavigationNext( instance, metadata.parameters.asBool("disableNextButton",false) );
			*/
		}

		// ToDo: turn feature back on once API method context.stage.navigation.logic.getNextPageAccess() is working
		// This feature has been disabled as long as the API method
		// context.stage.navigation.logic.getNextPageAccess()
		// is not yet working correctly. Next button, Evaluate button and Memory game can interact and block the
		// next button navigation. Without the mentioned method working, user is still able to navigate to next page
		// by using the arrow keys on the keyboard
		/*
		function setAllowNavigationNext( controlInstance, navigationControlActive ){
			var buttonNextControlGroupId = "tts_button_next_control_" + metadata.page.getId();
			if( navigationControlActive ) {
				context.page.group.event.fire(controlInstance, buttonNextControlGroupId, "onActivateButtonNext", controlInstance);
			}
		}

		function registerNavigationControlElement( controlInstance, navigationControlActive ){
			var buttonNextControlGroupId = "tts_button_next_control_" + metadata.page.getId();
			if( navigationControlActive ) {
				context.page.group.event.fire(controlInstance, buttonNextControlGroupId, "onDeactivateButtonNext", controlInstance);
			}
		}
		*/

		function createDOM(){

            var bgImage = setupShellElement();

            evaluateButton = createButton( bgImage );

            if (isInAuthoringMode) {
                return;
            }

            hideButton();
		}

        function bindApiEvents(){

            if (isInAuthoringMode) {
                return;
            }

            context.page.event.bindOnEnter(instance);
            context.page.quiz.event.bindOnEvaluateTestRun(instance);
            context.page.quiz.event.bindOnUserSolution(instance);
            context.page.quiz.event.bindOnBestPracticeSolution(instance);
        }


        // revealing public API
		return {
			onLoading: function () {
				instance = this;
                createDOM();
                bindApiEvents();
			},

			onEnterPage: function () {

				// ToDo: turn feature back on once API method context.stage.navigation.logic.getNextPageAccess() is working
				// This feature has been disabled as long as the API method
				// context.stage.navigation.logic.getNextPageAccess()
				// is not yet working correctly. Next button, Evaluate button and Memory game can interact and block the
				// next button navigation. Without the mentioned method working, user is still able to navigate to next page
				// by using the arrow keys on the keyboard
				/*
				registerNavigationControlElement( instance, metadata.parameters.asBool("disableNextButton",false) );
				*/

				if (!isStandardPlayer && !context.page.quiz.hasQuestion()) {
					hideButton();
					return;
				}

				if (isEvaluated()) {

					(currentMode === ButtonMode.GOTO_BEST_PRACTICE_SOLUTION) ? switchToUserSolution() : switchToBestPracticeSolution();

					if (isAssessmentMode) {
						showSubmitExceededButton();
					}

					return;
				}

                showEvaluateButton();
			},

			onExitPage: function() {
				removeEvents();
			},

			onEvaluateTestRun : function( dataHTML5 ){

                context.page.event.trigger(instance, "evaluation");
                if ( isStandardPlayer ) {
                    return;
                }

				if ( dataHTML5.isFinalized || dataHTML5.currentTestRun >= dataHTML5.testRuns) {
					context.page.quiz.switchToUserSolution();
				}
			},

			onUserSolution: switchToUserSolution,
			onBestPracticeSolution: switchToBestPracticeSolution
		};
	};
});
