// com.tts.initial.html5.quiz.ddtarget.autoform
define(["jquery"], function( jq ) {

	"use strict";

    function TargetElementBuilder( context, metadata ){

        var parameters = metadata.parameters,
            backgroundImage = createBackgroundImageModel(),
            fonts = [
                'Arial, Helvetica, sans-serif',
                '"Times New Roman", Times, serif',
                'Verdana, Geneva, sans-serif',
                '"Courier New", Courier, monospace',
                'Roboto, "Helvetica Neue", Helvetica, Arial, sans-serif',
                '"Droid Serif", "Times New Roman", Times, serif'
            ];

        function getFont( index ) {
            var fontIndex = (index > 0 && index < fonts.length) ? index-1: 0;
            return fonts[fontIndex];
        }

        function asRgbaValue( colorParameterName, defaultColor, transparencyParameterName ) {

            var color = hexToRgb( parameters.get(colorParameterName, defaultColor)),
                alpha = asAlpha( parameters.asNumber(transparencyParameterName, 0) );

            return "rgba(" + color.r + ", " + color.g + ", " + color.b + ", " + alpha + ")";
        }

        function hexToRgb( hex ) {
            var parsingResult = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
            return parsingResult ? {
                r: parseInt(parsingResult[1], 16),
                g: parseInt(parsingResult[2], 16),
                b: parseInt(parsingResult[3], 16)
            } : {
                r: 0,
                g: 0,
                b: 0
            };
        }

        function asAlpha( percentValue ) {
            return (100 - percentValue) / 100;
        }

        function createTarget(){

            var	fontSize = parameters.get("font-size", "14" ),
                borderSize = parameters.get("border-size", "1" ),
                target;

            target = jq(document.createElement("div"))
                .css({
                    "width" : "100%",
                    "height" : "100%",
                    "background-color" : asRgbaValue( "background-color", "#cccccc", "background-transparency" ),
                    "font-family" : getFont( parameters.asNumber("font-family", 0 ) ),
                    "font-size" : fontSize + "px",
                    "font-weight" : "bold",
                    "font-style" : "normal",
                    "color" : parameters.get("color", "#919191" ),
                    "line-height" : fontSize + "px",
                    "padding-left" : (parameters.asNumber("padding-left", "15") - parseInt(borderSize)) + "px",
                    "padding-right" : (parameters.asNumber("padding-right", "15") - parseInt(borderSize)) + "px",
                    "padding-top" : (parameters.asNumber("padding-top", "15") - parseInt(borderSize)) + "px",
                    "text-align" : parameters.get("text_orientation", "center"),
                    "box-sizing" : "border-box",
                    "border" : ( borderSize > 0 ) ? borderSize + "px solid " + asRgbaValue( "border-color", "#98a7b5", "border-transparency" ) : "0"
                })
                .html( parameters.get( "text", "" ) );

            backgroundImage.initialize( target );

            return target;
        }

        function createBackgroundImageModel() {

            var backgroundImage = parameters.get("bg_image", null),
                backgroundFileName = "",
                bgHeight = 0,
                bgLayer;

            if (backgroundImage) {
                backgroundFileName = context.stage.getSharedPath() + backgroundImage.fileName;
                bgHeight = backgroundImage.height / 2;
            }

            return {
                initialize : function( jqElement ){

                    if( !backgroundFileName || backgroundFileName.length === 0 || !jqElement ){
                        return;
                    }

                    bgLayer = jqElement;

                    bgLayer.css({
                        "background-image": "url(\"" + backgroundFileName + "\")",
                        "background-position" : "0px 0px"
                    });
                },
                setDefaultStyle : function(){
                    if( bgLayer ){
                        bgLayer.css("background-position", "0px 0px");
                    }
                },
                setDragOverTargetStyle : function(){
                    if( bgLayer ) {
                        bgLayer.css("background-position", "0px -" + bgHeight + "px" );
                    }
                }
            };
        }


        return {

    		build : function() {

                var targetDiv = createTarget();

    			return {
                    getTargetDiv : function() {
                        return targetDiv;
                    },
                    setDefaultStyle : backgroundImage.setDefaultStyle,
                    setDragOverTargetStyle : backgroundImage.setDragOverTargetStyle
                }
			}
		}
    }

    function OffsetService( context ){

        function getRelativeStageOffset( element ){

            var jqElement = jq(element),
                elementOffset = jqElement.offset(),
                stageOffset = context.stage.getOffset(),
                stageZoom = context.stage.getZoom(),
                zoomedOffset = {
                    left : (elementOffset.left - stageOffset.left) / stageZoom,
                    top : (elementOffset.top - stageOffset.top) / stageZoom
                };

            return zoomedOffset;
        }

        return {
            getRelativeStageOffset : getRelativeStageOffset
        }
    }

    function TargetRegistry( context, metadata ){

        var	variableNamespace = 'com.tts.dragAndDrop',
			stepId = metadata.page.getId(),
            targets = {};

        function register( target ){
            getTargets();
            targets[stepId][target.getId()] = target;
            setTargets();
        }

        function deregister( target ){
            getTargets();
            if( typeof(targets[stepId]) !== "undefined" ) {
                targets[stepId][target.getId()] = null;
                setTargets();
            }
        }

        function getTargets(){
            targets = context.stage.variable.get('DragAndDropTargetRegistry', variableNamespace ) || {};
            if( typeof(targets[stepId]) === "undefined" ) {
                targets[stepId] = {};
            }
            return targets;
        }

        function setTargets(){
            context.stage.variable.set('DragAndDropTargetRegistry', targets, variableNamespace );
        }

        return {
            register : register,
            deregister : deregister
        };
    }

    function SourceComponents( context, parameters ){

        var multipleSourcesAllowed = parameters.get("disallow_multiple_sources") !== "true",
            defaultDestinationOffset = {
                "left" : parameters.asNumber( "offset-left", 10 ),
                "top" : parameters.asNumber( "offset-top", 40 )
            },
            spacer = parameters.asNumber("entry-space", 8),
            sources = [],
            offsetService = new OffsetService(context);

        function isValidSource( source ){

            if( !source ){
                return false;
            }

            if( typeof(source.getHeight) !== "function" ){
                return false;
            }

            if( typeof(source.isCorrectAssignment) !== "function" ){
                return false;
            }

            if( typeof(source.onPositionOnTargetChanged) !== "function" ){
                return false;
            }

            return true;
        }

        function add(sourceObject) {

            if (!isValidSource(sourceObject)) {
                return false;
            }

            sources.push(sourceObject);
            return true;
        }

        function refreshPositions() {

            var positioningOffset = getSourceDestinationOffset();

            jq.each(sources, function (index, source) {
                source.onPositionOnTargetChanged( positioningOffset, index );
                positioningOffset.top += source.getHeight() + spacer;
            });
        }

        function getSourceDestinationOffset() {

            var shellElement = jq(context.getElement()),
                positioningOffset = offsetService.getRelativeStageOffset( shellElement );

            return {
                left : positioningOffset.left + defaultDestinationOffset.left,
                top : positioningOffset.top + defaultDestinationOffset.top
            };
        }

        return {

        	add: add,

            addByUser: function (sourceObject) {

                if (!add(sourceObject)) {
                    return;
                }

                refreshPositions();
            },

            remove: function (sourceObject) {

                var newSources = [];

                jq.each( sources, function (index, source) {
                    if (sourceObject.getId() !== source.getId()) {
                        newSources.push(source);
                    }
                });

                sources = newSources;
                refreshPositions();
            },

            removeAll: function () {
                sources = [];
            },

            isCapacityAvailable : function() {
            	return multipleSourcesAllowed || sources.length === 0;
        	},

        	addForSolution: function (sourceObject) {

                if (!add(sourceObject)) {
                    return;
                }

                refreshPositions();
            }
    	};
    }

    /**
	 * @constructor
	 */
	return function( context, metadata ) {

		// declare private members
		var self = null,
			parameters = metadata.parameters,

            isStandardPlayer = context.getPlayerName() === "standard",

			target,
			targetId = parameters.get( "target_id", "1"),

        	sourceComponents = new SourceComponents( context, parameters ),
			targetRegistry = new TargetRegistry( context, metadata );


		function createComponent() {

			self = this;

			setComponentBehaviour();

			target = new TargetElementBuilder( context, metadata ).build();

            var shellElement = jq(context.getElement());
            shellElement.css( getCoreCss() )
                .append( target.getTargetDiv() );

            if (context.stage.isInAuthoringMode()) {
                shellElement.prepend( createAuthoringShield() );
            }
        }

		function setComponentBehaviour(){
			context.page.event.bindOnEnter( self );
			context.page.event.bindOnExit( self );
            context.page.quiz.event.bindOnUserSolution( self );
            context.page.quiz.event.bindOnBestPracticeSolution( self );
		}

		function getCoreCss(){

            var coreStyles = {
				"cursor" : "default"
			};

            if( !isStandardPlayer ) {
                coreStyles["width"] = "100%";
                coreStyles["height"] = "100%";
		}

            return coreStyles;
		}

		function createAuthoringShield() {
			return jq( document.createElement("div") )
				.css({
					"width" : "100%",
					"height" : "100%",
					"position" : "absolute",
					"opacity" : "0.15",
					"background-image" : parameters.get("background-transparency", "0" ) === 100 ? "url(\"" + context.getPath() + "invis.png\")" : ""
				});
		}

		function initialize() {
            sourceComponents.removeAll();
            targetRegistry.register( self );
		}

		function cleanUp() {
			targetRegistry.deregister( self );
		}

		function onSourceDrop( sourceObject ) {

			if( !sourceComponents.isCapacityAvailable() ) {
				return false;
			}

            sourceComponents.addByUser( sourceObject );
            target.setDefaultStyle();

			return true;
		}

        function setAsUserSolution( sourceObject ) {
            sourceComponents.addForSolution( sourceObject );
        }

		function initializeUserSolution() {
            sourceComponents.removeAll();
		}

		function setAsBestPracticeSolution( sourceObject ) {

			if( !sourceComponents.isCapacityAvailable() ) {
				return false;
			}

            sourceComponents.addForSolution( sourceObject );
			return true;
		}

		function initializeBestPracticeSolution(){
            sourceComponents.removeAll();
		}

		// revealing public API
		return {

			onLoading : createComponent,
			onEnterPage : initialize,
			onExitPage : cleanUp,
			getId : function() {
				return metadata.getId();
			},
			getTargetId : function() {
				return targetId;
			},
            getPosition : function(){
				var shellElement = jq(context.getElement());
				return {
					offset :shellElement.offset(),
					size : {
						width : shellElement.width(),
                        height : shellElement.height()
					}
				};
			},
			onDragStart : function( sourceComponent ) {
                sourceComponents.remove( sourceComponent );
			},
			onDragOver : function() {
                target.setDragOverTargetStyle();
			},
			onDragOut : function() {
                target.setDefaultStyle();
			},
			onDrop : onSourceDrop,
            onUserSolution : initializeUserSolution,
            setAsUserSolution : setAsUserSolution,
            onBestPracticeSolution : initializeBestPracticeSolution,
            setAsBestPracticeSolution : setAsBestPracticeSolution
		};
	};
});
