define(["jquery"],function( jq ) {
	"use strict";

	/**
	 * @constructor
	 */
	function ScoreDetailed( context, metadata ) {

		var parameters = metadata.parameters,
			borderWidth = parameters.get("borderWidth", 1),
			isSectionEvaluation = parameters.asBool("isSectionEvaluation" , "0"),
			alternateColors = parameters.asBool("alternateColors", "0"),
			backgroundColor1 = asRgbaValue("backgroundColor1", "#d8dee3", "backgroundColor1Transparency"),
			backgroundColor2 = asRgbaValue("backgroundColor2", "#e2e6ea", "backgroundColor2Transparency"),
			borderColor = asRgbaValue("borderColor", "#98a7b5", "borderTransparency"),
			borderStyle = parameters.get( "borderStyle" , "horizontal"),
			imageCorrect = parameters.get("iconCorrect", null),
			iconImageCorrect = imageCorrect? context.stage.getSharedPath() + imageCorrect.fileName : context.getPath() + "correct.png",
			imageWrong = parameters.get("iconWrong", null),
			iconImageWrong = imageWrong? context.stage.getSharedPath() + imageWrong.fileName : context.getPath() + "wrong.png",
			col1Css = {"text-align": "right", "padding-right": "5px", "width": parameters.get("widthCol1", 30)},
			col3Css = {"text-align": "center", "padding": "0 5px", "width": parameters.get("widthCol3", 100)},
			col4Css = {"text-align": "right", "width": "1px"},
			currentPage = metadata.page,
			scoringSectionsA = [],
			allPagesA = metadata.stage.pages,
			aQuestions = context.quiz.getQuestions(),
			isTouchDevice = ('ontouchstart' in window) || (navigator.MaxTouchPoints > 0) || (navigator.msMaxTouchPoints > 0) || context.device.isMobile();

		function asRgbaValue( colorParameterName, defaultColor, transparencyParameterName ) {
			var color = hexToRgb( parameters.get(colorParameterName, defaultColor)),
				alpha = asAlpha( parameters.asNumber(transparencyParameterName, 0) );

			return "rgba(" + color.r + ", " + color.g + ", " + color.b + ", " + alpha + ")";
		}

		function hexToRgb( hex ) {
			var parsingResult = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
			return parsingResult ? {
				r: parseInt(parsingResult[1], 16),
				g: parseInt(parsingResult[2], 16),
				b: parseInt(parsingResult[3], 16)
			} : {
				r: 0,
				g: 0,
				b: 0
			};
		}

		function asAlpha( percentValue ) {
			return (100 - percentValue) / 100;
		}

		function createView() {
			var shellElement = jq(context.getElement())
				.css({
					"font-family" : "Arial, Helvetica, sans-serif",
					"font-size" : "12px",
					"font-weight" : "normal",
					"height" : metadata.getHeight() + "px"
				});

			shellElement.append(createResultTable());
			if (context.stage.isInAuthoringMode()) {
				createAuthoringShield(shellElement);
			}
		}

		function refreshView() {
			jq(context.getElement()).html(createResultTable());
		}

		function createResultTable() {
			var resultTable = jq(document.createElement("table"))
				.css({
					"width": "100%",
					"border-spacing": "0",
					"border-collapse": "collapse",
					"border-top": borderWidth + "px solid " + borderColor
				});

			if (borderStyle !== "horizontal") {
				resultTable.css({
					"border-right": borderWidth + "px solid " + borderColor,
					"border-left": borderWidth + "px solid " + borderColor
				});
			}

			resultTable.append(createResultTableBody());

			return createScrollWrapper( resultTable );
		}

		function createScrollWrapper( table ) {

			var overflowWrapper = jq( document.createElement("div") );
			overflowWrapper.css({
				"width" : "100%",
				"height" : "100%",
				"overflow-x" : "visible",
				"overflow-y" : "auto"
			});
			overflowWrapper.append(table);

			if( isTouchDevice ){
				addTouchScroll( overflowWrapper );
			}

			return overflowWrapper;
		}

		function addTouchScroll( jqElement ) {

			var scrollStartPos = 0,
				positionData;

			jqElement.on( "touchstart", function (event) {
				positionData = getPointerPositionFromEvent( event );

				if( jqElement.get(0).scrollHeight > jqElement.height() ) {
					lockNavigation();
				}

				scrollStartPos = this.scrollTop + positionData.pageY;
				event.preventDefault();
			});

			jqElement.on( "touchmove", function (event) {
				positionData = getPointerPositionFromEvent( event );
				this.scrollTop = scrollStartPos - positionData.pageY;
				event.preventDefault();
			});

			jqElement.on( "touchend", function (event) {
				unlockNavigation();
				event.preventDefault();
			});
		}

		function getPointerPositionFromEvent( event ){

			if( event.originalEvent ) {

				if( event.originalEvent.touches && event.originalEvent.touches[0] ){
					return event.originalEvent.touches[0];
				}

				if( event.originalEvent.changedTouches && event.originalEvent.changedTouches[0] ){
					return event.originalEvent.changedTouches[0];
				}
			}

			return event;
		}

		function lockNavigation() {
			if( context.stage.trigger && typeof( context.stage.trigger.lock ) === "function" ) {
				context.stage.trigger.lock(metadata.getId());
			}
		}

		function unlockNavigation() {
			if( context.stage.trigger && typeof( context.stage.trigger.unlock ) === "function" ) {
				context.stage.trigger.unlock(metadata.getId());
			}
		}

		function createResultTableBody() {
			var tBody = jq(document.createElement("tbody")),
				rows = getResultTableRows();

			rows.forEach(function(row) {
				tBody.append(row);
			});

			return tBody;
		}

		function getResultTableRows() {
			if (context.stage.isInAuthoringMode()) {
				return getAuthoringTableRows();
			}

			return getTableRows();
		}

		function getAuthoringTableRows() {
			var tableRows = [],
				authoringTable = {
					rows: [
						// row1
						{
							id: "quizRow_0",
							backgroundColor: backgroundColor1,
							cells: [
								{ type: "text", value: "1", css: col1Css},
								{ type: "text", value: "Quiz 1"},
								{ type: "text", value: "5 / 5", css: col3Css},
								{ type: "image", value: iconImageCorrect, css: col4Css }
							]
						},
						// row2
						{
							id: "quizRow_1",
							backgroundColor: alternateColors? backgroundColor2 : backgroundColor1,
							cells: [
								{ type: "text", value: "2", css: col1Css },
								{ type: "text", value: "Quiz 2" },
								{ type: "text", value: "0 / 5", css: col3Css },
								{ type: "image", value: iconImageWrong, css: col4Css }
							]
						}
					]
				};

			authoringTable.rows.forEach(function(row) {
				tableRows.push(createTableRow(row));
			});

			return tableRows;
		}

		function getQuestion(page){
			for (var i=0; i < aQuestions.length; i++) {
				if (page.getId() === aQuestions[i].getPageMetadata().getId()) {
					return aQuestions[i];
				}
			}
			return null;
		}

		function getTableRows() {
			var rowsCollection = [],
				tableRows = [],
				question,
				questionNumber = 0,
				resultObject,
				backgroundColor,
				relevantPages = getRelevantQuizPages(),
				allQuizPages = allPagesA.filter(function(page) {
					return getQuestion(page) !== null;
				});

			relevantPages.forEach(function(page) {
				question = getQuestion(page);
				questionNumber++;

				resultObject = getQuestionResultObject(question);
				backgroundColor = (alternateColors && questionNumber%2 === 0)? backgroundColor2 : backgroundColor1;

				rowsCollection.push({
					id: "quizRow_" + questionNumber,
					backgroundColor: backgroundColor,
					cells: [
						{ type: "text", value: allQuizPages.indexOf(page)+1, css: col1Css },
						{ type: "text", value: page.getTitle() },
						{ type: "text", value: resultObject.userScore + " / " + resultObject.maxScore, css: col3Css },
						{ type: "image", value: resultObject.icon, css: col4Css }
					]
				});
			});

			rowsCollection.forEach(function(rowObject) {
				tableRows.push(createTableRow(rowObject));
			});

			return tableRows;
		}

		function getRelevantQuizPages() {
			var startPageIndex = getLastSectionPageIndex() + 1,
				stopPageIndex = allPagesA.indexOf(currentPage),
				allRelevantPages = allPagesA.slice(startPageIndex, stopPageIndex),
				relevantQuizPages;

			if (!isSectionEvaluation) {
				allRelevantPages = allPagesA;
			}

			relevantQuizPages = allRelevantPages.filter(function(page) {
				return getQuestion(page) !== null;
			});

			return relevantQuizPages;
		}

		function getQuestionResultObject(question) {
			var userScore = parseInt(question.getScore()),
				maxScore = parseInt(question.getMaxScore()),
				icon = (userScore > 0 && userScore === maxScore) ? iconImageCorrect : iconImageWrong;

			return {
				userScore : userScore,
				maxScore : maxScore,
				icon : icon
			}
		}

		function getLastSectionPageIndex() {
			if (!isSectionEvaluation || scoringSectionsA.length === 0) {
				return 0;
			}

			var lastSectionPageId = null,
				lastSectionPage,
				id;

			for(id in scoringSectionsA) {
				if (scoringSectionsA.hasOwnProperty(id)) {
					if (scoringSectionsA[id] === currentPage.getId()) {
						break;
					}
					lastSectionPageId = scoringSectionsA[id];
				}
			}

			lastSectionPage = getPage(lastSectionPageId);
			if (!lastSectionPageId) {
				return 0;
			}

			return allPagesA.indexOf(lastSectionPage);
		}

		function getPage(id) {
			var page = null;

			if (!id) {
				return null;
			}

			for (var p in allPagesA) {
				if (allPagesA.hasOwnProperty(p) && allPagesA[p].getId() === id) {
					return allPagesA[p];
				}
			}

			return page;
		}

		function createTableRow(rowObject) {
			var row = jq(document.createElement("tr"))
				.attr("id", rowObject.id)
				.css("backgroundColor", rowObject.backgroundColor);

			rowObject.cells.forEach(function(cellObject) {
				row.append(createTableCell(cellObject));
			});

			return row;
		}

		function createTableCell(cellObject) {
			var cell = jq(document.createElement("td"))
				.css({
					"-moz-background-clip" : "padding-box",
					"-webkit-background-clip" : "padding-box",
					"background-clip" : "padding-box",
					"border-bottom" : borderWidth + "px solid "+ borderColor
				});

			if (cellObject.css) {
				cell.css(cellObject.css);
			}

			switch(cellObject.type) {
				case "text":
					cell.text(cellObject.value);
					break;
				case "image":
					cell.append(createImage(cellObject.value));
					break;
				default:
					break;
			}

			return cell;
		}

		function createImage(source) {
			return jq(document.createElement("img")).attr("src", source);
		}


		function createAuthoringShield( shellElement ) {
			// create an object shield so we are able to select and drag the element in the editor
			var objShield = jq(document.createElement("div"));
			objShield.css({
				"position" : "absolute",
				"left" : "0px",
				"top" : "0px",
				"background-color" : "#ffffff",
				"opacity" : "0",
				"width" : "100%",
				"height" : "100%",
				"z-index" : "9999"
			});
			shellElement.append(objShield);
		}

		// revealing public API
		return {
			onLoading : function() {
				context.page.event.bindOnEnter( this );
				var scoringSectionsJSON = context.stage.variable.get( "tts_scoreSectionId", "tts.scoreDetailed" );
				if (scoringSectionsJSON){
					scoringSectionsA = jq.parseJSON( scoringSectionsJSON );
				}
				context.stage.variable.set( "tts_scoreSectionId", JSON.stringify( scoringSectionsA ), "tts.scoreDetailed" );

				createView();
			},

			onEnterPage : function() {
				var currentPageId = currentPage.getId(),
					tmpPagesIds = scoringSectionsA;

				if (tmpPagesIds.indexOf(currentPageId) === -1) {
					tmpPagesIds.push(currentPageId);
				}

				scoringSectionsA = [];
				allPagesA.forEach(function(page) {
					if (tmpPagesIds.indexOf(page.getId()) !== -1) {
						scoringSectionsA.push(page.getId());
					}
				});

				context.stage.variable.set( "tts_scoreSectionId", JSON.stringify( scoringSectionsA ), "tts.scoreDetailed" );
				refreshView();
			}
		};
	}

	return ScoreDetailed;
});