// com.tts.initial.html5.sound.onoff
// HTML4 / HTML5: No differences
define(["jquery", "./locals"],
    function (jq, uiText) {

    "use strict";

    var EventUtil = (function() {

        var
            isIOS = (/iphone|ipad|ipod/i.test( navigator.userAgent.toLowerCase() ) );

        function getFirstSupportedEvent(tagName, eventNames) {
            var result = "";

            jq.each(eventNames.split(" "), function (i, evtName) {
                if (isEventSupported(evtName, tagName)) {
                    result = evtName;
                    return false;
                }
            });
            return result;
        }

        function isEventSupported(eventName, tagName) {
            var elem = tagName ? document.createElement(tagName) : document,
                onEventName = "on" + eventName,
                isSupported;

            isSupported = !!(onEventName in elem);
            if (!isSupported && typeof elem.setAttribute !== "undefined") {
                elem.setAttribute(onEventName, "return;");
                isSupported = typeof elem[onEventName] === "function";
            }

            return isSupported;
        }

        return {
            /**
             * Attach the event handler <b>handler</b> for the first supported event specified by <b>eventNames</b>
             * to the element <b>jqObject</b>.
             * Example: attachFirstSupportedEvent( myObject, "pointerdown mousedown touchend", myHandler );
             *
             * @param element
             * @param {String} eventNames
             * @param handler
             * @return {String} the event name or ""
             */
            attachFirstSupportedEvent: function (element, eventNames, handler) {
                var jqObject = jq(element),
                    evtName = getFirstSupportedEvent(element.nodeName, eventNames);

                if (!evtName) {
                    return "";
                }

                jqObject.on(evtName, handler);
                return evtName;
            },

            /**
             * Remove attached event handler <b>handler</b> for the first supported event specified by <b>eventNames</b>
             * from the element <b>jqObject</b>.
             * Example: removeFirstSupportedEvent( myObject, "pointerdown mousedown touchend", myHandler );
             *
             * @param element
             * @param {String} eventNames
             * @param handler
             * @return {String} the event name or ""
             */
            removeFirstSupportedEvent: function (element, eventNames, handler) {
                var jqObject = jq(element),
                    evtName = getFirstSupportedEvent(element.nodeName, eventNames);

                if (!evtName) {
                    return "";
                }

                jqObject.off(evtName, handler);
                return evtName;
            },

            getPointerDownEvent: function( element ) {
                if( !isIOS ) {
                    return getFirstSupportedEvent( element.nodeName, "pointerdown mousedown touchstart" );
                }

                // iOS 9.3 unterstützt zwar click und mouseup, weshalb sich auf diese Events attached wird, aber sie werden
                //  aus unbekannten Gründen nicht gesendet.
                return "touchstart";
            },

            getPointerUpEvent: function( element ) {

                if( !isIOS ) {
                    return getFirstSupportedEvent( element.nodeName, "click pointerup mouseup touchend" );
                }

                // iOS 9.3 unterstützt zwar click und mouseup, weshalb sich auf diese Events attached wird, aber sie werden
                //  aus unbekannten Gründen nicht gesendet.
                return "touchend";
            }
        };
    })();

	var ButtonState = {
			Default : "btn_state_default",
            Hover : "btn_state_hover",
            Pressed : "btn_state_pressed",
            Inactive : "btn_state_inactive",

		},
        ButtonToggleState = {
            On : "btn_state_on",
            Off : "btn_state_off"
        },

        getClassName = function( toggleState, buttonState ){
            return toggleState + "_" + buttonState;
        },

        ImageButtonBuilder = function( options ){

            var backgroundPositions = {};

            backgroundPositions[ ButtonToggleState.On ] = {};
            backgroundPositions[ ButtonToggleState.On ][ ButtonState.Default ] = "0 0";
            backgroundPositions[ ButtonToggleState.On ][ ButtonState.Hover ] = "0 " + (-options.height) + "px";
            backgroundPositions[ ButtonToggleState.On ][ ButtonState.Pressed ] = "0 " + (-2 * options.height) + "px";
            backgroundPositions[ ButtonToggleState.On ][ ButtonState.Inactive ] = "0 " + (-3 * options.height) + "px";

            backgroundPositions[ ButtonToggleState.Off ] = {};
            backgroundPositions[ ButtonToggleState.Off ][ ButtonState.Default ] = "-" + options.width + "px 0";
            backgroundPositions[ ButtonToggleState.Off ][ ButtonState.Hover ] = "-" + options.width + "px " + (-options.height) + "px";
            backgroundPositions[ ButtonToggleState.Off ][ ButtonState.Pressed ] = "-" + options.width + "px " + (-2 * options.height) + "px";
            backgroundPositions[ ButtonToggleState.Off ][ ButtonState.Inactive ] = "-" + options.width + "px " + (-3 * options.height) + "px";

            function createTextContent() {

                if (jq.trim(options.text.content) === "") {
                    return null;
                }

                var textContainer = "";
                if (options.text.verticalAlignment === "middle") {
                    textContainer = jq(document.createElement("div"))
                        .css({
                            "width": "auto",
                            "padding-right": options.text.paddingRight + "px",
                            "padding-left": options.text.paddingLeft + "px",
                            "vertical-align": "middle",
                            "display": "table-cell"
                        });
                }
                else {
                    textContainer = jq(document.createElement("div"))
                        .css({
                            "position": "absolute",
                            "padding": options.text.paddingTop + "px " + options.text.paddingRight + "px " + options.text.paddingBottom + "px " + options.text.paddingLeft + "px"
                        });

                    if (options.text.verticalAlignment === "bottom" || options.text.verticalAlignment === "top" ) {
                        textContainer.css(options.text.verticalAlignment, "0");
                    }

                    if( options.text.orientation === "right" ) {
                        textContainer.css(options.text.orientation, "0");
                    }
                }

                textContainer.html(options.text.content);
                return textContainer;
            }

            function createButton() {

                var currentState = ButtonState.Default,
                    currentToggleState = ButtonToggleState.On,
                    textContent = createTextContent(),
                    imageButton = jq(document.createElement("div"))
                        .attr({
                            "title" : options.caption,
                            "class" : currentState
                        })
                        .css({
                        "width": options.width + "px",
                        "height": options.height + "px",
                        "text-align": options.text.orientation,
                        "font-family": options.text.fontFamily,
                        "color": options.color,
                        "-webkit-tap-highlight-color": "rgba(0,0,0,0)",
                        "font-size": Math.min( Math.abs(options.text.fontSize), 100) + "px",
                        "cursor": "pointer"
                    })
                    .addClass("sc-navigation")
                    .addClass("ttsButtonSoundOnOff")
                    .on("touchmove", function(e) {
                        e.preventDefault ? e.preventDefault() : true;
                    });

                if (options.text.verticalAlignment === "middle") {
                    imageButton.css("display", "table");
                }

                if( textContent ) {
                    imageButton.append(textContent);
                }

                return {
                    getHotspotElement : function(){
                        return imageButton.get(0);
                    },
                    changeImage : function( width, height, filename ){

                        backgroundPositions[ ButtonToggleState.On ][ ButtonState.Hover ] = "0 " + (-height) + "px";
                        backgroundPositions[ ButtonToggleState.On ][ ButtonState.Pressed ] = "0 " + (-2 * height) + "px";
                        backgroundPositions[ ButtonToggleState.On ][ ButtonState.Inactive ] = "0 " + (-3 * height) + "px";
                        backgroundPositions[ ButtonToggleState.Off ][ ButtonState.Default ] = "-" + width + "px 0";
                        backgroundPositions[ ButtonToggleState.Off ][ ButtonState.Hover ] = "-" + width + "px " + (-height) + "px";
                        backgroundPositions[ ButtonToggleState.Off ][ ButtonState.Pressed ] = "-" + width + "px " + (-2 * height) + "px";
                        backgroundPositions[ ButtonToggleState.Off ][ ButtonState.Inactive ] = "-" + width + "px " + (-3 * height) + "px";

                        jq(imageButton).css({
                            "width": width + "px",
                            "height": height + "px",
                            "background-image": "url(\"" + filename + "\")",
                            "background-position": backgroundPositions[ currentToggleState ][ currentState ]
                        });
                    },
                    hasState : function( buttonState ){
                        return buttonState === currentState;
                    },
                    hasToggleState : function( buttonToggleState ){
                        return buttonToggleState === currentToggleState;
                    },
                    changeState : function( buttonState, toggleState ){

                        if( !toggleState ){
                            toggleState = currentToggleState;
                        }

                        if( !buttonState ){
                            buttonState = currentState;
                        }

                        var enabled = buttonState !== ButtonState.Inactive,
                            oldClassName = getClassName( currentToggleState, currentState ),
                            newClassName = getClassName( toggleState, buttonState );

                        imageButton.removeClass( oldClassName );
                        currentState = buttonState;
                        currentToggleState = toggleState;
                        imageButton.addClass( newClassName );
                        imageButton.css({
                            "background-position": backgroundPositions[toggleState][buttonState],
                            "cursor": enabled ? "pointer" : "default"
                        });
                    }
                };
            }

			return {
				build : createButton
			};
		},
		SvgButtonBuilder = function( options ){

			var buttonColors = {};

            buttonColors[ ButtonToggleState.On ] = {};
            buttonColors[ ButtonToggleState.On ][ ButtonState.Default ] = options.colors.default;
            buttonColors[ ButtonToggleState.On ][ ButtonState.Hover ] = options.colors.hover;
            buttonColors[ ButtonToggleState.On ][ ButtonState.Pressed ] = options.colors.pressed;
            buttonColors[ ButtonToggleState.On ][ ButtonState.Inactive ] = options.colors.inactive;

            buttonColors[ ButtonToggleState.Off ] = {};
            buttonColors[ ButtonToggleState.Off ][ ButtonState.Default ] = options.colors.default;
            buttonColors[ ButtonToggleState.Off ][ ButtonState.Hover ] = options.colors.hover;
            buttonColors[ ButtonToggleState.Off ][ ButtonState.Pressed ] = options.colors.pressed;
            buttonColors[ ButtonToggleState.Off ][ ButtonState.Inactive ] = options.colors.inactive;

            function createSvgPath(){

                return  '<svg id="' + options.id + '" width="' + options.width + 'px" height="' + (options.height * 4) + 'px" viewBox="0 0 44 176" preserveAspectRatio="none" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" style="background-color: transparent;">' +
                            '<title>' + options.caption +'</title>' +
                            '<defs></defs>' +
                            '<g stroke="none" stroke-width="1" fill="none" fill-rule="evenodd">' +
                                '<g>' +
                                    '<g class="NavigationButton" transform="translate(0.000000, 0.000000)" fill="' + options.colors.default.bg + '">' +
                                        '<rect x="0" y="0" width="44" height="44"></rect>' +
                                        '<path class="NavigationButtonIcon_On" display="block" fill="' + options.colors.default.icon + '" d="M10,17.4712117 L15.5643778,17.4712117 L22.5689468,10.3199997 L22.5689468,33.1771426 L15.5643778,26.0259317 L10,26.0259317 L10,17.4712117 Z M24.8542099,17.1771426 L27.139473,17.1771426 L27.139473,26.3199997 L24.8542099,26.3199997 L24.8542099,17.1771426 Z M29.424736,13.7485711 L31.7099991,13.7485711 L31.7099991,29.7485711 L29.424736,29.7485711 L29.424736,13.7485711 Z" id="Combined-Shape" fill-rule="nonzero"></path>' +
                                        '<path class="NavigationButtonIcon_Off" display="none" fill="' + options.colors.default.icon + '" d="M15.534968,17.4859471 L10,11.9509792 L11.9509792,10 L34,32.0490208 L32.0490208,34 L22.5714286,24.5224077 L22.5714286,33.191878 L15.5654765,26.0406671 L10,26.0406671 L10,17.4859471 L15.534968,17.4859471 Z M18.8467355,14.1366551 L22.5714286,10.3347351 L22.5714286,17.8613482 L18.8467355,14.1366551 Z M27.1428571,22.4327768 L24.8571429,20.1470625 L24.8571429,17.191878 L27.1428571,17.191878 L27.1428571,22.4327768 Z M31.7142857,27.0042053 L29.4285714,24.718491 L29.4285714,13.7633065 L31.7142857,13.7633065 L31.7142857,27.0042053 Z" id="Combined-Shape" fill-rule="nonzero"></path>' +
                                    '</g>' +
                                '</g>' +
                            '</g>' +
						'</svg>';
            }

            function createButton(){

                var currentState = ButtonState.Default,
                    currentToggleState = ButtonToggleState.On,
                    svgButton = jq(document.createElement("div")),
                    svg = svgButton.html( createSvgPath() )
                    .contents()
                    .addClass("sc-navigation")
                    .addClass("ttsButtonSoundOnOff")
                    .css({
                        "pointer-events" : "none"
                    })
                    .on("touchmove", function(e) {
                        e.preventDefault ? e.preventDefault() : true;
                    })
                    .get(0);

                var svgWrapper = jq(document.createElement("div"))
                    .attr({
                        "title" : options.caption,
                        "class" : getClassName( currentToggleState, currentState )
                    })
                    .css({
                        "cursor": "pointer",
                        "width": options.width + "px",
                        "height": options.height + "px",
                        "overflow": "hidden"
                    })
                    .append(svg);

                return {
                    getHotspotElement : function(){
                        return svgWrapper.get(0);
                    },
                    hasState : function( buttonState ){
                      return buttonState === currentState;
                    },
                    hasToggleState : function( buttonToggleState ){
                        return buttonToggleState === currentToggleState;
                    },
                    changeState : function( buttonState, toggleState ){

                        if( !toggleState ){
                            toggleState = currentToggleState;
                        }

                        if( !buttonState ){
                            buttonState = currentState;
                        }

                        var colors = buttonColors[toggleState][buttonState],
                            jqSvg = jq(svg),
                            oldClassName = getClassName( currentToggleState, currentState ),
                            newClassName = getClassName( toggleState, buttonState );

                        svgWrapper.removeClass( oldClassName );
                        currentToggleState = toggleState;
                        currentState = buttonState;
                        svgWrapper.addClass( newClassName );

                        if( toggleState === ButtonToggleState.On ) {
                            jqSvg.find("[class=NavigationButtonIcon_Off]").attr("display", "none");
                            jqSvg.find("[class=NavigationButtonIcon_On]").attr("display", "block");
                        }
                        else {
                            jqSvg.find("[class=NavigationButtonIcon_On]").attr("display", "none");
                            jqSvg.find("[class=NavigationButtonIcon_Off]").attr("display", "block");
                        }

                        jqSvg.find("[class=NavigationButton]").attr( "fill", colors.bg );
                        jqSvg.find("[class=NavigationButtonIcon]").attr( "fill", colors.icon );
                        jqSvg.css({
                            "cursor" : buttonState !== ButtonState.Inactive ? "pointer" : "default"
                        });
                    }
                };
            }

            return {
                build : createButton
            };
        };


    /**
     * @constructor
     */
    return function (context, metadata) {
        var shellElement,
			enabled = true,
            button,
            buttonInitialized = false,
            isStandardPlayer = context.getPlayerName() === "standard",
            hasStageTriggerSupport = true,
            isInAuthoringMode = false,
            pointerUpEventName,
            pointerDownEventName;

        function loadBackgroundImage() {

            var deferred = jq.Deferred(),
                defaultImage = new Image(),
                metadataParameters = metadata.parameters,
                backgroundImage = metadataParameters.get("bg_image", null),
                backgroundFileName,
                result = {};

            defaultImage.onload = function() {
                result.width = this.width / 2;
                result.height = this.height / 4;
                deferred.resolve(result);
            };

            if (backgroundImage) {
                backgroundFileName = context.stage.getSharedPath() + backgroundImage.fileName;
            }
            else {
                backgroundFileName = context.getPath() + "sound_on_off.png";
            }

            result.fileName = backgroundFileName;
            defaultImage.src = backgroundFileName;

            return deferred.promise();
        }

        // create an object shield so we are able to select and drag element
        function createAuthoringShield(shellElement) {
            shellElement.append(jq(document.createElement("div"))
                .css({
                    "position": "absolute",
                    "left": 0,
                    "top": 0,
					"background-color": "#ffffff",
                    "opacity": "0.4",
                    "width": "100%",
                    "height": "100%",
                    "z-index": "9999"
                }));
        }

        function lockNavigation() {
			if (isInAuthoringMode || !hasStageTriggerSupport) {
                return;
            }
            context.stage.trigger.lock(metadata.getId());
        }

        function unlockNavigation() {
			if (isInAuthoringMode || !hasStageTriggerSupport) {
                return;
            }
            context.stage.trigger.unlock(metadata.getId());
        }

        function onMouseOver() {
            if( enabled && !button.hasState( ButtonState.Pressed ) ) {
                button.changeState( ButtonState.Hover, getCurrentAudioToggleState() );
            }
		}

        function onMouseLeave() {
            unlockNavigation();
            if( enabled ) {
                button.changeState( ButtonState.Default, getCurrentAudioToggleState() );
			}
        }

        function onMouseDown() {
            lockNavigation();
            button.changeState( ButtonState.Pressed, getCurrentAudioToggleState() );
        }

        function getCurrentAudioToggleState(){
            return context.stage.audio.isOn() ? ButtonToggleState.On : ButtonToggleState.Off;
        }

        function isOnTarget(e) {

            var elem = e.target,
                origEvent = e.originalEvent,
                changedTouch = origEvent.changedTouches && origEvent.changedTouches.length > 0 ? origEvent.changedTouches[0] : undefined;

            if(changedTouch) {
                elem = document.elementFromPoint(changedTouch.clientX, changedTouch.clientY);
            }
            return e.target === elem;
        }

        function onMouseUp(e) {

            unlockNavigation();

            if( !enabled ) {
                return;
            }

            toggleAudioState();

            if(isOnTarget(e)) {
                button.changeState( ButtonState.Hover, getCurrentAudioToggleState() );
            }
            else {
                button.changeState( ButtonState.Default, getCurrentAudioToggleState() );
            }
        }

        function toggleAudioState(){
            if( context.stage.audio.isOn() ){
                context.stage.audio.off();
            }
            else {
                context.stage.audio.on();
            }
        }

        function initPointerEventNames() {
            if( !pointerUpEventName ) {
                pointerUpEventName = EventUtil.getPointerUpEvent( button.getHotspotElement() );
                pointerDownEventName = EventUtil.getPointerDownEvent( button.getHotspotElement() );
            }
        }

        function bindEvents() {

            if (context.stage.isInAuthoringMode()) {
                return;
            }

            if (!button || !buttonInitialized) {
                return;
            }

            removeEvents();
            EventUtil.attachFirstSupportedEvent( button.getHotspotElement(), pointerDownEventName, onMouseDown );
            EventUtil.attachFirstSupportedEvent( button.getHotspotElement(), "pointermove mousemove", onMouseOver );
            EventUtil.attachFirstSupportedEvent( button.getHotspotElement(), "pointerout mouseout", onMouseLeave );
            EventUtil.attachFirstSupportedEvent( button.getHotspotElement(), pointerUpEventName, onMouseUp );
        }

        function removeEvents() {
            unlockNavigation();
            if (!button || !buttonInitialized) {
                return;
            }

            initPointerEventNames();
            EventUtil.removeFirstSupportedEvent( button.getHotspotElement(), pointerDownEventName, onMouseDown );
            EventUtil.removeFirstSupportedEvent( button.getHotspotElement(), "pointermove mousemove", onMouseOver );
            EventUtil.removeFirstSupportedEvent( button.getHotspotElement(), "pointerout mouseout", onMouseLeave );
            EventUtil.removeFirstSupportedEvent( button.getHotspotElement(), pointerUpEventName, onMouseUp );
        }

        function initializeButton(){

        	if( buttonInitialized ){
        		return;
			}
            buttonInitialized = true;

            shellElement.css({
                "width": metadata.getWidth() + "px",
                "height": metadata.getHeight() + "px"
                });

            button = metadata.parameters.asBool("use_svg") ? createSvgButton() : createImageButton();
            shellElement.append( button.getHotspotElement() );

            if (isInAuthoringMode) {
                createAuthoringShield( shellElement );
            }

            button.changeState( ButtonState.Default, getCurrentAudioToggleState() );
            }

        function createImageButton(){

            var scParams = metadata.parameters,
                imageButton = new ImageButtonBuilder({
                    width: metadata.getWidth(),
                    height: metadata.getHeight(),
                    caption: uiText.get("player.sound.onoff", metadata.stage.getContentLanguage()),
                    color : scParams.get("color", "#919191"),
                    text : {
                        content : scParams.get("text", ""),
                        fontFamily : getFontStyle( scParams.asNumber("font-family", "1") ),
                        fontSize : scParams.asNumber("font-size", "14"),
                        orientation : scParams.get("text_orientation", "center"),
                        verticalAlignment: scParams.get("text_vertical_align", "middle"),
                        paddingLeft : Math.abs(scParams.asNumber("padding-left", "10")),
                        paddingRight : Math.abs(scParams.asNumber("padding-right", "10")),
                        paddingTop : Math.abs(scParams.asNumber("padding-top", "10")),
                        paddingBottom : Math.abs(scParams.asNumber("padding-bottom", "10"))
                    }
                }).build();

            loadBackgroundImage().then( function( buttonImage ) {
                applyButtonImage( buttonImage );
                });

            return imageButton;
        }

        function createSvgButton() {

            var scParams = metadata.parameters,
                pageId = metadata.page.getId();

            return new SvgButtonBuilder({
                id: "svg_" + pageId + "_" + metadata.getId(),
                width: metadata.getWidth(),
                height: metadata.getHeight(),
                caption: uiText.get("player.sound.onoff", metadata.stage.getContentLanguage()),
                colors: {
                    default: {
                        icon: scParams.get("color_svg_icon_default", "#ffffff"),
                        bg: scParams.get("color_svg_background_default", "#707070")
                    },
                    hover: {
                        icon: scParams.get("color_svg_icon_hover", "#ffffff"),
                        bg: scParams.get("color_svg_background_hover", "#fd7e2f")
                    },
                    pressed: {
                        icon: scParams.get("color_svg_icon_pressed", "#ffffff"),
                        bg: scParams.get("color_svg_background_pressed", "#df7a3b")
                    },
                    inactive: {
                        icon: scParams.get("color_svg_icon_inactive", "#a9a9a9"),
                        bg: scParams.get("color_svg_background_inactive", "#707070")
            }
        }
            }).build();
        }

        function getFontStyle( paramValue ) {
            switch( paramValue ) {
                case 2:
                    return "\"Times New Roman\", Times, serif";
                case 3:
                    return "Verdana, Geneva, sans-serif";
                case 4:
                    return "\"Courier New\", Courier, monospace";
                default:
                    return "Arial, Helvetica, sans-serif";
            }
        }

        function applyButtonImage( buttonImage ){

            var buttonWidth = buttonImage.width,
                buttonHeight = buttonImage.height;

            shellElement.css({
                "width": buttonWidth + "px",
                "height": buttonHeight + "px"
            });

            button.changeImage( buttonWidth, buttonHeight, buttonImage.fileName );
        }

        function isFirstPage() {

            if( !metadata.page ){
                return false;
            }

            var currentPageId = metadata.page.getId(),
                firstPage = getFirstPageInLearningPath( currentPageId );

            return firstPage && firstPage.getId() === currentPageId;
        }

        function getFirstPageInLearningPath( currentPageId ){

            if( typeof metadata.page.getLearningPath === "function") {
                var currentLearningPath = metadata.page.getLearningPath(currentPageId);
                return currentLearningPath ? currentLearningPath.pages[0] : null;
            }

            if(metadata.stage && metadata.stage.pages) {
                return metadata.stage.pages[0];
            }

            return null;
        }

        function isMainLearningPath() {
            var currentLearningPath = (isStandardPlayer) ? metadata.page.getLearningPath() : metadata.stage,
                isMainPath = true;

            try {
                if (typeof currentLearningPath.isMainLearningPath === "function") {
                    isMainPath = currentLearningPath.isMainLearningPath();
                }
            }
            catch (err) {
                isMainPath = true;
            }

            return isMainPath;
        }

        // revealing public API
        return {
            onLoading: function () {
                context.page.event.bindOnEnter(this);
                context.page.event.bindOnExit(this);
                context.stage.event.bindOnAudioControlChanged(this);
                hasStageTriggerSupport = !!context.stage.trigger;
                isInAuthoringMode = !!context.stage.isInAuthoringMode();

                shellElement = jq(context.getElement());

                // context.stage.audio.isOn() Api method returns wrong initial state on pages
                // where no sound objects are placed on, until it is set once manually.
                // so if we are on first page, we set it to on once
                if( isFirstPage() && isMainLearningPath() ) {
                    var soundStateHasBeenInitialized = context.stage.variable.get('soundOnOffInitialStateHack', 'tts.soundOnOffButton');
                    if( !soundStateHasBeenInitialized ) {
                        context.stage.audio.on();
                        context.stage.variable.set('soundOnOffInitialStateHack', true, 'tts.soundOnOffButton');
                    }
                }

                // because of some problems with loading SVG while it is hidden on page,
                // we initialize SVG button onEnterPage
                if (!isStandardPlayer || isInAuthoringMode) {
                    initializeButton();
                }
            },

            onEnterPage: function () {

                // because of some problems with loading SVG while it is hidden on page,
                // we initialize SVG button onEnterPage
				if( isStandardPlayer ) {
                    initializeButton();
                }

                bindEvents();

				unlockNavigation();

            },
            onExitPage: function() {
                removeEvents();
            },
            onAudioControlChanged: function() {
                button.changeState( null, getCurrentAudioToggleState() );
            }
        };
    };
});
